import sys; sys.path.insert(0, "../..")
from pyx import *

def bracepath(w1, w2, h=0.1, slant=0.1):
    p1 = path.path(path.moveto(-w1, -h),
                   path.lineto(-w1+slant, 0),
                   path.lineto(-slant, 0),
                   path.lineto(0, h))
    p2 = path.path(path.moveto(0, h),
                   path.lineto(slant, 0),
                   path.lineto(w2-slant, 0),
                   path.lineto(w2, -h))
    p1 = deformer.smoothed(10*h).deform(p1)
    p2 = deformer.smoothed(10*h).deform(p2)
    p = p1 << p2
    p = deformer.smoothed(0.1*h).deform(p)
    return p.split([0.05, p.end()-0.05])[1]

def brace(c, x, y, w1, w2):
    # This is a well known trick to draw a line with an eliptic pen.
    # you just stroke the path with a circular pen and transform the
    # result. Since we only do a horizontal scaling here, the widths
    # w1 and w2 don't need to be adjusted.
    sc = canvas.canvas([trafo.translate(x, y), trafo.scale(1, 2)])
    c.insert(sc)
    sc.stroke(bracepath(w1, w2), [style.linecap.round])

linespacing = 1.4*text.text(0, 0, "X").height

class person(canvas.canvas):

    def __init__(self, lines, childs=[],
                       distance=0.5, relpos=0.5):
        canvas.canvas.__init__(self)
        self.width = 0
        y = 0
        for line in lines:
            t = text.text(0, y, line, [text.halign.center])
            if t.width > self.width:
                self.width = t.width
            self.insert(t)
            y -= linespacing
        if childs:
            width = sum([child.width for child in childs], 0)
            width += distance*(len(childs)-1)
            brace(self, 0, y, relpos*width, (1-relpos)*width)
            x = -relpos*width
            for child in childs:
                x += 0.5*child.width
                self.insert(child, [trafo.translate(x, y-2*linespacing)])
                x += 0.5*child.width + distance

p = person([r"Otto III.", r"* 1215 -- $\dagger$ 1267"],
           [person(["Johann III."]),
            person(["Otto V. der Lange"],
                   [person(["Otto"]),
                    person(["Albrecht"]),
                    person(["Hermann"]),
                    person(["Beatrix", "etc."])],
                   relpos=0.8),
            person(["Albrecht III."],
                   [person(["Otto"]),
                    person(["Johann"]),
                    person(["Beatrix"]),
                    person(["Margarette"])],
                   relpos=0.2),
            person(["Otto IV. der Kleine"]),
            person(["Kunigunde"])])
p.writeEPSfile("genealogy")



import math, random
from pyx import *

# a xy-graph has linear x and y axes by default
# they might be overwritten and further axes might be added as well
g = graph.graphxy(width=8, y=graph.axis.log(), y2=graph.axis.lin(),
                  y3=graph.axis.lin(min=0, max=1),
                  y4=graph.axis.lin(min=0, max=2))

# we generate some data and a function with multiple arguments
d = [[i, math.exp(0.8*i+random.random())] for i in range(1,10)]
f = lambda x, a: x*a

g.plot(graph.data.points(d, x=1, y=2))
g.plot(graph.data.function("y2(x)=f(x, 1)", context=locals()))

g.plot(graph.data.function("x(y3)=5+sin(2*pi*y3)"))
g.plot(graph.data.function("x(y4)=5+sin(2*pi*y4)"))

g.writeEPSfile()
g.writePDFfile()
g.writeSVGfile()



#!/usr/bin/env python
# -*- coding: ISO-8859-15 -*-
import sys
sys.path.insert(0, "..")

import math
from math import *
from pyx import *
from pyx.graph import axis, data

# define some functions for the color values:
def red(x):   return 2*x*(1-x)**5 + 3.5*x**2*(1-x)**3 + 2.1*x*x*(1-x)**2 + 3.0*x**3*(1-x)**2 + x**0.5*(1-(1-x)**2)
def green(x): return 1.5*x**2*(1-x)**3 - 0.8*x**3*(1-x)**2 + 2.0*x**4*(1-x) + x**4
def blue(x):  return 5*x*(1-x)**5 - 0.5*x**2*(1-x)**3 + 0.3*x*x*(1-x)**2 + 5*x**3*(1-x)**2 + 0.5*x**6

# use the implemented converter for the grey value:
def grey(x): return color.rgb(red(x), green(x), blue(x)).grey().color["gray"]

# plot the color values
g = graph.graphxy(width=10, ratio=1)
g.plot(data.function("y(x)=red(x)",   min=0, max=1, context=locals()), [graph.style.line([color.rgb.red])])
g.plot(data.function("y(x)=green(x)", min=0, max=1, context=locals()), [graph.style.line([color.rgb.green])])
g.plot(data.function("y(x)=blue(x)",  min=0, max=1, context=locals()), [graph.style.line([color.rgb.blue])])
g.plot(data.function("y(x)=grey(x)",  min=0, max=1, context=locals()))

# plot the colors
n = 200
x = 0
for i in range(n):
    t = i * 1.0 / (n-1)
    g.fill(path.rect(x, -2, 10.5/(n-1), 1), [color.rgb(red(t), green(t), blue(t))])
    x += 10.0 / n

g.writeEPSfile("palettetest", paperformat=document.paperformat.A4)
g.writePDFfile("palettetest")




#!/usr/bin/env python
import sys; sys.path[:0] = ["../.."]

import math
from pyx import *
from pyx.graph.axis.parter import linear as linparter
from pyx.graph.axis.painter import regular, ticklength, rotatetext
from pyx.graph.axis.texter import rational, default
from pyx.graph.axis.axis import lin, pathaxis

c = canvas.canvas()
lintest = {"title": "axis title", "min": 0, "max": 1, "parter": linparter(["0.25", "0.1/0.8"])}
c.insert(pathaxis(path.path(path.moveto(0, 0), path.lineto(0, 8)),
                        lin(**lintest),
                        direction=-1))
c.insert(pathaxis(path.path(path.moveto(1, 0), path.lineto(1, 8)),
                        lin(**lintest)))
c.insert(pathaxis(path.path(path.moveto(5, 0), path.lineto(5, 8)),
                        lin(painter=regular(labelattrs=[trafo.rotate(45)], titleattrs=[trafo.rotate(45)]), **lintest),
                        direction=-1))
c.insert(pathaxis(path.path(path.moveto(8, 0), path.lineto(8, 8)),
                        lin(painter=regular(labelattrs=[trafo.rotate(45), text.halign.right], titleattrs=[trafo.rotate(-45)]), **lintest),
                        direction=-1))
c.insert(pathaxis(path.path(path.moveto(11, 0), path.lineto(11, 8)),
                        lin(painter=regular(tickattrs=[color.rgb.red], innerticklength=0, outerticklength=ticklength.normal), **lintest),
                        direction=-1))
c.insert(pathaxis(path.path(path.moveto(12, 0), path.lineto(12, 8)),
                        lin(painter=regular(tickattrs=[attr.changelist([None, color.rgb.green])]), **lintest)))
c.insert(pathaxis(path.path(path.moveto(16, 0), path.lineto(16, 8)),
                        lin(texter=default(), **lintest),
                        direction=-1))
c.insert(pathaxis(path.path(path.moveto(18, 0), path.lineto(18, 8)),
                        lin(texter=rational(), **lintest),
                        direction=-1))
lintest = {"title": "axis title", "min": -2*math.pi, "max": 0, "divisor": math.pi, "parter": linparter(["0.25"])}
c.insert(pathaxis(path.path(path.moveto(0, 11), path.lineto(8, 11)),
                        lin(texter=rational(suffix=r"\pi"), **lintest)))
lintest = {"title": "axis title", "min": 0, "max": 2*math.pi, "divisor": math.pi, "parter": linparter(["0.5"])}
c.insert(pathaxis(path.path(path.moveto(10, 11), path.lineto(18, 11)),
                        lin(texter=rational(numsuffix=r"\pi", over="%s/%s"), **lintest)))
lintest = {"min": 0, "max": 2*math.pi, "divisor": math.pi, "parter": linparter(["0.125"])}
c.insert(pathaxis(path.circle(4, 17, 4),
                        lin(texter=rational(suffix=r"\pi"), **lintest)))
lintest = {"min": 0, "max": 2*math.pi, "divisor": math.pi/180, "parter": linparter(["30"])}
c.insert(pathaxis(path.circle(14, 17, 4),
                        lin(painter=regular(labeldirection=rotatetext.parallel), **lintest)))
c.writeEPSfile("test_axis", page_paperformat=document.paperformat.A4)
c.writePDFfile("test_axis", page_paperformat=document.paperformat.A4)
c.writeSVGfile("test_axis", page_paperformat=document.paperformat.A4)




# contributed by Michael Schindler

from pyx import *

# get the lines from the graph
xax = graph.axis.linear(min=-1, max=1.0, painter=None)
yax = graph.axis.linear(min=-1.3, max=1.3, painter=None)
g = graph.graphxy(width=10, ratio=2, x=xax, y=yax)
pifline = g.plot(graph.data.function("y(x)=sin(1.0/(x**2+0.02122))", points=1000))
pihoriz = g.plot(graph.data.function("y(x)=0.5*x", points=2))
g.finish()

# fetch path from info returned by plot method of graph
fline = pifline.path
horiz = pihoriz.path

# intersect the lines
splith, splitf = horiz.intersect(fline)

# create gray area (we do not use simple clipping)
area = horiz.split([splith[0]])[0]
for i in range(0, len(splith)-2, 2):
    area = area.joined(fline.split([splitf[i], splitf[i+1]])[1])
    area = area.joined(horiz.split([splith[i+1], splith[i+2]])[1])
area = area.joined(fline.split([splitf[-2], splitf[-1]])[1])
area = area.joined(horiz.split([splith[-1]])[1])
area.append(path.lineto(*g.vpos(1, 0)))
area.append(path.lineto(*g.vpos(0, 0)))
area.append(path.closepath()) # not really needed (filling closes automatically)

c = canvas.canvas()

# draw first the area, then the function
c.fill(area, [color.gray(0.6)])
c.stroke(fline, [style.linewidth.Thick, style.linejoin.round])

c.writeEPSfile()
c.writePDFfile()
c.writeSVGfile()



import sys
if sys.path[0] != "../..":
    sys.path.insert(0, "../..")

import unittest

from pyx import *
from pyx.text import MultiEngineText
from pyx.graph.axis.tick import tick
from pyx.graph.axis.texter import rational, decimal, default, skipmantissaunity


def tex_expr(label):
    if isinstance(label, MultiEngineText):
        return label.tex
    return label


class TexterTestCase(unittest.TestCase):

    def testFrac(self):
        ticks = [tick((1, 4), labellevel=0), tick((2, 4), labellevel=0)]
        rational(numsuffix=r"\pi").labels(ticks)
        self.assertEqual([tex_expr(t.label) for t in ticks], [r"{{\pi}\over{4}}", r"{{\pi}\over{2}}"])
        ticks = [tick((0, 3), labellevel=0), tick((3, 3), labellevel=0), tick((6, 3), labellevel=0)]
        rational(numsuffix=r"\pi").labels(ticks)
        self.assertEqual([tex_expr(t.label) for t in ticks], ["0", r"\pi", r"2\pi"])
        ticks = [tick((2, 3), labellevel=0), tick((4, 5), labellevel=0)]
        rational(numsuffix=r"\pi", equaldenom=1).labels(ticks)
        self.assertEqual([tex_expr(t.label) for t in ticks], [r"{{10\pi}\over{15}}", r"{{12\pi}\over{15}}"])

    def testDec(self):
        ticks = [tick((1, 4), labellevel=0), tick((2, 4), labellevel=0)]
        decimal().labels(ticks)
        self.assertEqual([t.label for t in ticks], ["0.25", "0.5"])
        ticks = [tick((1, 4), labellevel=0), tick((2, 4), labellevel=0)]
        decimal(equalprecision=1).labels(ticks)
        self.assertEqual([t.label for t in ticks], ["0.25", "0.50"])
        ticks = [tick((1, 17), labellevel=0), tick((17, 17), labellevel=0)]
        decimal().labels(ticks)
        self.assertEqual([t.label for t in ticks], [r"0.\overline{0588235294117647}", "1"])
        ticks = [tick((1, 10000000), labellevel=0), tick((1, 100000000), labellevel=0), tick((1, 1000000000), labellevel=0)]
        decimal(thousandthpartsep=",").labels(ticks)
        self.assertEqual([t.label for t in ticks], ["0.000,000,1", "0.000,000,01", "0.000,000,001"])
        ticks = [tick((1000000, 1), labellevel=0), tick((10000000, 1), labellevel=0), tick((100000000, 1), labellevel=0)]
        decimal(thousandsep=",").labels(ticks)
        self.assertEqual([t.label for t in ticks], ["1,000,000", "10,000,000", "100,000,000"])

    def testDefault(self):
        ticks = [tick((0, 10), labellevel=0), tick((1, 10), labellevel=0), tick((1, 1), labellevel=0), tick((10, 1), labellevel=0)]
        default().labels(ticks)
        self.assertEqual([tex_expr(t.label) for t in ticks], ["0", "0.1", "1", "10"])
        ticks = [tick((0, 10), labellevel=0), tick((1, 10), labellevel=0), tick((1, 1), labellevel=0), tick((10000, 1), labellevel=0)]
        default(minexponent=1, skipmantissaunity=skipmantissaunity.never, uniformexponent=False).labels(ticks)
        self.assertEqual([tex_expr(t.label) for t in ticks], [r"0", r"1\cdot{}10^{-1}", r"1", r"1\cdot{}10^{4}"])
        ticks = [tick((0, 10), labellevel=0), tick((1, 10), labellevel=0), tick((1, 1), labellevel=0), tick((10000, 1), labellevel=0)]
        default(minexponent=2, uniformexponent=False).labels(ticks)
        self.assertEqual([tex_expr(t.label) for t in ticks], ["0", "0.1", "1", r"10^{4}"])
        ticks = [tick((-1, 10), labellevel=0), tick((1, 1), labellevel=0), tick((10, 1), labellevel=0)]
        default(minexponent=0).labels(ticks)
        self.assertEqual([tex_expr(t.label) for t in ticks], [r"-10^{-1}", r"10^{0}", r"10^{1}"])
        ticks = [tick((0, 1), labellevel=0), tick((1, -10), labellevel=0), tick((15, 100), labellevel=0)]
        default(minnegexponent=0, mantissatexter=decimal(equalprecision=True)).labels(ticks)
        self.assertEqual([tex_expr(t.label) for t in ticks], [r"0.0\cdot{}10^{0}", r"-1.0\cdot{}10^{-1}", r"1.5\cdot{}10^{-1}"])
        ticks = [tick((0, 1), labellevel=0), tick((1, -10), labellevel=0), tick((15, 100), labellevel=0)]
        default(minnegexponent=1, mantissatexter=decimal(equalprecision=True)).labels(ticks)
        self.assertEqual([tex_expr(t.label) for t in ticks], [r"0.0", r"-1.0\cdot{}10^{-1}", r"1.5\cdot{}10^{-1}"])


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
import sys; sys.path[:0] = ["../.."]

from pyx import *

text.set(text.UnicodeEngine)

c = canvas.canvas()
c.text(0, 2, "Hello, world!")
c.text(0, 4, "Hello, world!", [color.rgb.red])
c.text(0, 6, "Hello, world!", [trafo.rotate(90)])

c.writeEPSfile()
c.writePDFfile()
c.writeSVGfile()



import sys; sys.path.insert(0, "../..")
import math
from pyx import *

a = 1
p = path.curve(-1, 0, 1, a, -1, a, 1, 0)

t = trafo.rotate(30).scaled(5)
x = 0.49

p = p.transformed(t)

c = canvas.canvas()
c.stroke(p)
t = p.normsubpaths[0].trafo([x])[0]
if t is not path.invalid:
    c.stroke(path.line(0, 0, 1, 0), [p.normsubpaths[0].trafo([x])[0], deco.earrow.normal, color.rgb.red])

c.text(-6, -5, "t=%f" % x)
c.text(-6, -5.5, "$x(t)=%f$" % (2.54/72*p.normsubpaths[0].normsubpathitems[0].x_pt(x)))
c.text(-6, -6, "$\dot x(t)=%f$" % (2.54/72*p.normsubpaths[0].normsubpathitems[0].xdot_pt(x)))
c.text(-6, -6.5, "$\ddot x(t)=%f$" % (2.54/72*p.normsubpaths[0].normsubpathitems[0].xddot_pt(x)))
c.text(-3, -5.5, "$y(t)=%f$" % (2.54/72*p.normsubpaths[0].normsubpathitems[0].y_pt(x)))
c.text(-3, -6, "$\dot y(t)=%f$" % (2.54/72*p.normsubpaths[0].normsubpathitems[0].ydot_pt(x)))
c.text(-3, -6.5, "$\ddot y(t)=%f$" % (2.54/72*p.normsubpaths[0].normsubpathitems[0].yddot_pt(x)))
c.text(0, -6, "datan2$(\dot y(t), \dot x(t))=%f$" % (math.atan2(2.54/72*p.normsubpaths[0].normsubpathitems[0].ydot_pt(x), 2.54/72*p.normsubpaths[0].normsubpathitems[0].xdot_pt(x))*180/math.pi))
c.text(0, -6.5, "datan2$(\ddot y(t), \ddot x(t))=%f$" % (math.atan2(2.54/72*p.normsubpaths[0].normsubpathitems[0].yddot_pt(x), 2.54/72*p.normsubpaths[0].normsubpathitems[0].xddot_pt(x))*180/math.pi))

g = c.insert(graph.graphxy(width=10, xpos=-5, ypos=-20, key=graph.key.key(), y=graph.axis.lin(min=-10, max=10)))
g.plot(graph.data.functionxy(lambda t: 2.54/72*p.normsubpaths[0].normsubpathitems[0].y_pt(t), title="$y(t)$", min=0, max=1))
g.plot(graph.data.functionxy(lambda t: 2.54/72*p.normsubpaths[0].normsubpathitems[0].ydot_pt(t), title="$\dot y(t)$", min=0, max=1))
g.plot(graph.data.functionxy(lambda t: 2.54/72*p.normsubpaths[0].normsubpathitems[0].yddot_pt(t), title="$\ddot y(t)$", min=0, max=1))
g.stroke(g.xgridpath(x), [color.rgb.red])
g.stroke(g.ygridpath(0))

g = c.insert(graph.graphxy(width=10, xpos=-5, ypos=0.5+g.ypos+g.height, key=graph.key.key(), x=graph.axis.linkedaxis(g.axes["x"]), y=graph.axis.lin(min=-10, max=10)))
g.plot(graph.data.functionxy(lambda t: 2.54/72*p.normsubpaths[0].normsubpathitems[0].x_pt(t), title="$x(t)$", min=0, max=1))
g.plot(graph.data.functionxy(lambda t: 2.54/72*p.normsubpaths[0].normsubpathitems[0].xdot_pt(t), title="$\dot x(t)$", min=0, max=1))
g.plot(graph.data.functionxy(lambda t: 2.54/72*p.normsubpaths[0].normsubpathitems[0].xddot_pt(t), title="$\ddot x(t)$", min=0, max=1))
g.stroke(g.xgridpath(x), [color.rgb.red])
g.stroke(g.ygridpath(0))

c.writeEPSfile("cusp")



# contributed by Gert-Ludwig Ingold

from pyx import *

n = 3                 # number of masses
r = 3.0               # system radius
rcyc = 0.3            # radius of cycloid
nl = 13               # number of loops
rc = 0.5              # radius of masses
eps = 0.03            # extra spacing for surrounding circles

c = canvas.canvas()
springcircle = path.circle(0, 0, r)
masspositions = [i*springcircle.arclen()/n
                 for i in range(n)]
for springsegment in springcircle.split(masspositions):
    c.stroke(springsegment,
             [deformer.cycloid(rcyc, nl),
              deformer.smoothed(radius=0.1)])
for x, y in springcircle.at(masspositions):
    c.fill(path.circle(x, y, rc))

c.stroke(springcircle, [deformer.parallel(rc+eps)])
c.stroke(springcircle, [deformer.parallel(-rc-eps)])

c.writeEPSfile()
c.writePDFfile()
c.writeSVGfile()



from pyx import *

c = canvas.canvas()

rect1 = path.path(path.moveto(0, 0), path.lineto(1, 0),
                  path.moveto(1, 0), path.lineto(1, 1),
                  path.moveto(1, 1), path.lineto(0, 1),
                  path.moveto(0, 1), path.lineto(0, 0))

rect2 = path.path(path.moveto(0, 0), path.lineto(0, 1),
                  path.lineto(1, 1), path.lineto(1, 0),
                  path.lineto(0, 0))

rect3 = path.path(path.moveto(0, 0), path.lineto(0, 1),
                  path.lineto(1, 1), path.lineto(1, 0),
                  path.closepath())

c.stroke(rect1, [trafo.scale(2), style.linewidth.THICK])
c.text(1, -0.7, "(a)", [text.halign.center])

c.stroke(rect2, [trafo.scale(2).translated(4, 0), style.linewidth.THICK])
c.text(5, -0.7, "(b)", [text.halign.center])

c.stroke(rect3, [trafo.scale(2).translated(8, 0), style.linewidth.THICK])
c.text(9, -0.7, "(c)", [text.halign.center])

c.stroke(rect3, [trafo.scale(2).translated(12, 0), style.linewidth.THICK, deco.filled([color.grey(0.5)])])
c.text(13, -0.7, "(d)", [text.halign.center])

c.writePDFfile()



#!/usr/bin/env python
import sys; sys.path[:0] = ["../.."]

from pyx import *
from pyx.path import *
from pyx import normpath

def bboxrect(cmd):
    return cmd.bbox().rect()

def dotest(c, x, y, test):
   c2 = c.insert(canvas.canvas([trafo.translate(x, y)]))
   eval("%s(c2)" % test)
   c.stroke(bboxrect(c2))

class cross(path):
   def __init__(self, x, y):
       path.__init__(self, moveto(x,y),
                           rmoveto(-0.1, -0.1),
                           rlineto(0.2, 0.2),
                           rmoveto(-0.1, -0.1),
                           rmoveto(-0.1, +0.1),
                           rlineto(0.2, -0.2))


def testarcs(c):
    def testarc(c, x, y, phi1, phi2):
        p=path(arc(x,y, 0.5, phi1, phi2))
        np = p.normpath()
        c.stroke(p, [color.rgb.red])
        c.stroke(np, [color.rgb.green, style.linestyle.dashed])

    def testarcn(c, x, y, phi1, phi2):
        p=path(arcn(x,y, 0.5, phi1, phi2))
        np = p.normpath()
        c.stroke(p, [color.rgb.red])
        c.stroke(np, [color.rgb.green, style.linestyle.dashed])

    def testarct(c, r, x0, y0, dx1, dy1, dx2, dy2):
        p=path(moveto(x0,y0), arct(x0+dx1,y0+dy1, x0+dx2, y0+dy2, r), rlineto(dx2-dx1, dy2-dy1), closepath())
        np = p.normpath()
        c.stroke(p, [color.rgb.red, style.linewidth.Thick])
        c.stroke(np, [color.rgb.green, style.linewidth.THin, deco.filled([color.rgb.green])])

    testarc(c, 1, 2, 0, 90)
    testarc(c, 2, 2, -90, 90)
    testarc(c, 3, 2, 270, 90)
    testarc(c, 4, 2, 90, -90)
    testarc(c, 5, 2, 90, 270)
    testarc(c, 4, 3, 45, -90)
    testarc(c, 2, 3, 45, -90-2*360)
    testarc(c, 1, 3, 45, +90+2*360)

    testarcn(c, 1, 5, 0, 90) 
    testarcn(c, 2, 5, -90, 90) 
    testarcn(c, 3, 5, 270, 90) 
    testarcn(c, 4, 5, 90, -90) 
    testarcn(c, 5, 5, 90, 270) 
    testarcn(c, 4, 6, 45, -90) 
    testarcn(c, 2, 6, 45, -90-360) 
    testarcn(c, 1, 6, 45, -90+360)


    x0 = -5
    for angle in [-45, 0, 45, 90, 135, 180, 225, 315]:
        angle_rad = angle*pi/180
        testarct(c, 0.1, x0, 0, 0, 1, 2*cos(angle_rad), 2*sin(angle_rad))
        x0 += 2

    testarct(c, 0.5, 1, 8, 0, 1, 1, 1)
    testarct(c, 0.5, 3, 8, 1, 1, 1, 2)
    testarct(c, 0.5, 5, 8, 1, 0, 2, 1)
    testarct(c, 0.5, 7, 8, 1, 0, 2, 0)
    testarct(c, 0.0, 9, 8, 0, 1, 1, 1)

#    testarct(c, 0.5, 11, 8, 0, 1, 0, 0) # not allowed


def testintersectbezier(c):
    p=path(moveto(0,0), curveto(2,6,4,5,2,9)).normpath(epsilon=1e-4)
    q=path(moveto(2,0), curveto(2,6,4,12,1,6)).normpath(epsilon=1e-4)

    c.stroke(q, [style.linewidth.THIN])
    c.stroke(p, [style.linewidth.THIN])

    isect = p.intersect(q)

    for i in isect[0]:
        x, y = p.at(i)
        c.stroke(cross(x, y), [style.linewidth.THIN])

def testintersectcircle(c):
    k=circle(0, 0, 2)
    l=line(0, 0, 3, 0)
    c.stroke(k, [style.linewidth.THIN])
    c.stroke(l, [style.linewidth.THIN])

    isect = k.intersect(l)
    assert len(isect[0])==1, "double count of intersections"

    for i in isect[0]:
        x, y = k.at(i)
        c.stroke(cross(x, y), [style.linewidth.THIN])

def testintersectline(c):
    l1=line(0, 0, 1, 1)
    l2=line(0, 1, 1, 0)
    c.stroke(l1, [style.linewidth.THIN])
    c.stroke(l2, [style.linewidth.THIN])

    isect = l1.intersect(l2)

    for i in isect[0]:
        x, y = l1.at(i)
        c.stroke(circle(x, y, 0.1), [style.linewidth.THIN])

    l1=curve(0, 0, 0, 0, 0, 0, 1, 1)
    c.stroke(l1, [style.linewidth.THIN])

    isect = l1.intersect(l2)

    for i in isect[0]:
        x, y = l1.at(i)
        c.stroke(circle(x, y, 0.1), [style.linewidth.THIN])


def testnormpathtrafo(c):
    p = path(moveto(0, 5),
             curveto(2, 1, 4, 0, 2, 4),
             rcurveto(-3, 2, 1, 2, 3, 6),
             rlineto(0, 3),
             closepath())

    c.stroke(p)
    c.stroke(p.normpath(), [color.rgb.green, style.linestyle.dashed])
    c.stroke(p, [trafo.translate(3, 1), color.rgb.red])
    c.insert(canvas.canvas([trafo.translate(3,1)])).stroke(p,
                                                          [color.rgb.green,
                                                          style.linestyle.dashed])

    c.stroke(p.reversed(), [color.rgb.blue, style.linestyle.dotted, style.linewidth.THick])

    c.stroke(cross(*(p.at(0))))
    c.stroke(cross(*(p.reversed().at(0))))

    p1, p2 = p.split([1.0, 2.1])
    c.stroke(p1, [color.rgb.red, style.linestyle.dashed])
    c.stroke(p2, [color.rgb.green, style.linestyle.dashed])

    circ1 = circle(0, 10, 1)
    circ2 = circle(1.7, 10, 1)

    c.stroke(circ1)
    c.stroke(circ2)

    isectcirc1, isectcirc2 = circ1.intersect(circ2)
    segment1 = circ1.split(isectcirc1)[0]
    segment2 = circ2.split(isectcirc2)[1]

    segment = segment1 << segment2
    segment[-1].close()

    c.stroke(segment, [style.linewidth.THick, deco.filled([color.rgb.green])])

def testtangent(c):
    p=path(moveto(0,5),
           curveto(2,1,4,0,2,4),
           rcurveto(-3,2,1,2,3,6),
           rlineto(2,3)) + circle(5,5,1)
    c.stroke(p, [style.linewidth.THick])
    arclen = p.arclen()
    points = 20
    for i in range(points):
        c.stroke(p.tangent(arclen*i/points, length=20*unit.t_pt), [color.rgb.blue, deco.earrow.normal])
        c.stroke(line(0, 0, 1, 0).transformed(p.trafo(arclen*i/points)), [color.rgb.green, deco.earrow.normal])
        c.stroke(line(0, 0, 0, 1).transformed(p.trafo(arclen*i/points)), [color.rgb.red, deco.earrow.normal])

    # test the curvature
    cc = canvas.canvas()
    cc.stroke(p)
    cc = canvas.canvas([canvas.clip(cc.bbox().path())])
    for i in range(points):
        curvature_pt = p.curvature_pt(unit.topt(arclen)*i/points)
        if curvature_pt != 0:
            radius_pt = 1/curvature_pt
            x_pt, y_pt = p.trafo(arclen*i/points).apply_pt(0, radius_pt)
            cc.stroke(circle_pt(x_pt, y_pt, radius_pt), [color.grey(0.5)])
    c.insert(cc)


def testarclentoparam(c):
    curve=path(moveto(0,0), lineto(0,5), curveto(5,0,0,10,5,5), closepath(),
               moveto(5,0), lineto(10,5))
    ll = curve.arclen()
    # l=[-0.8*ll, -0.6*ll, -0.4*ll, -0.2*ll, 0, 0.1*ll, 0.3*ll, 0.5*ll, 0.7*ll, 0.9*ll]
    l=[0, 0.1*ll, 0.2*ll, 0.3*ll, 0.4*ll, 0.5*ll, 0.6*ll, 0.7*ll, 0.8*ll, 0.9*ll]
    cols=[color.gray.black, color.gray(0.3), color.gray(0.7), color.rgb.red,
          color.rgb.green, color.rgb.blue, color.cmyk(1,0,0,0),
          color.cmyk(0,1,0,0), color.cmyk(0,0,1,0), color.gray.black]
    t=curve.arclentoparam(l)
    c.stroke(curve)
    for i in range(len(t)):
        c.draw(circle(curve.at(t[i])[0], curve.at(t[i])[1], 0.1), [deco.filled([cols[i]]), deco.stroked()])


c = canvas.canvas()
dotest(c, 0, 0, "testarcs")
dotest(c, 2, 12, "testintersectbezier")
dotest(c, 10,11, "testnormpathtrafo")
dotest(c, 12, -4, "testtangent")
dotest(c, 5, -4, "testintersectcircle")
dotest(c, 9, -4, "testintersectline")
dotest(c, 21, 12, "testarclentoparam")
c.writeEPSfile("test_path", page_paperformat=document.paperformat.A4, page_rotated=0, page_fittosize=1)
c.writePDFfile("test_path")
c.writeSVGfile("test_path")



from pyx import *

circ1 = path.circle(0, 0, 1.5)
circ2 = path.circle(1, 1, 1)
(circ1a, circ1b), (circ2a, circ2b) = circ1.intersect(circ2)
intersection = (circ2.split([circ2b, circ2a])[1]
                << circ1.split([circ1a, circ1b])[1])
intersection[-1].close()

union = (circ1.split([circ1a, circ1b])[0]
         << circ2.split([circ2b, circ2a])[0])
union[-1].close()

c = canvas.canvas()
c.fill(union, [color.rgb.blue])
c.fill(intersection, [color.rgb.red])
c.stroke(circ1)
c.stroke(circ2)
c.writeEPSfile()
c.writePDFfile()
c.writeSVGfile()



#!/usr/bin/env python
import sys; sys.path[:0] = ["../.."]

from pyx import *

text.set(text.LatexEngine)
c = canvas.canvas()
text.preamble(r"""%
    \usepackage{graphicx}
    \usepackage{color}
    \usepackage{rotating}
    \usepackage{helvet}
%
    \definecolor{col0}{gray}{0.1}
    \definecolor{col1}{cmyk}{0.3, 0.2, 0.1, 0.1}
    \definecolor{col2}{rgb}{0.4, 0.3, 0.1}
    \definecolor{col3}{RGB}{200, 200, 200}
    \definecolor{col4}{hsb}{0.1, 0.1, 0.1}
    \definecolor{col5}{named}{Red}
    \definecolor{col6}{pyx}{pyx.color.cmyk.PineGreen}
    \definecolor{col7}{pyx}{color.cmyk(0.92, 0, 0.59, 0.25)}
    \definecolor{col0}{gray}{0.5}""")

c.stroke(path.line(-1, 0, 6, 0))

c.stroke(path.line(6, 5, 6.99, 5), [style.linewidth.THIN])
c.stroke(path.line(6, 6, 6.99, 6), [style.linewidth.THIN])
c.stroke(path.line(8.01, 5, 9, 5), [style.linewidth.THIN])
c.stroke(path.line(8.01, 6, 9, 6), [style.linewidth.THIN])
c.stroke(path.line(7, 4, 7, 4.99), [style.linewidth.THIN])
c.stroke(path.line(8, 4, 8, 4.99), [style.linewidth.THIN])
c.stroke(path.line(7, 6.01, 7, 7), [style.linewidth.THIN])
c.stroke(path.line(8, 6.01, 8, 7), [style.linewidth.THIN])
c.text(7, 5, "\\vrule width1truecm height1truecm")

c.text(6.2, 0, "0", [text.vshift.middlezero])
c.text(-1.2, 0, "abc", [text.vshift.mathaxis, text.halign.right])

t1 = text.text(0, 0, r"a \PyXMarker{beforeb}b\PyXMarker{afterb} c d e f g h i j k l m n o p q r s t u v w x y z", [text.parbox(2, baseline=text.parbox.bottom)])
c.insert(t1)
c.stroke(t1.path())

t2 = c.insert(text.text(3, 0, r"a \PyXMarker{beforeb}b\PyXMarker{afterb} c d e f g h i j k l m n o p q r s t u v w x y z", [text.parbox(2, baseline=text.parbox.top)]))
c.stroke(t2.path())
c.stroke(path.line(*(t1.marker("beforeb") + t2.marker("beforeb"))), [color.rgb.red])
c.stroke(path.line(*(t1.marker("afterb") + t2.marker("afterb"))), [color.rgb.green])

c.text(0, 3, r"\int\limits_{-\infty}^\infty \!{\rm d}x\, e^{-a x^2} = \sqrt{\pi\over a}", [text.mathmode])
c.text(0, 6, r"\int\limits_{-\infty}^\infty \!{\rm d}x\, e^{-a x^2} = \sqrt{\pi\over a}", [text.size.LARGE, text.mathmode])

c.stroke(c.text(1, 2, r"Hello, world!").path())

# test a virtual font with encoding
c.text(0, 8, r"\sffamily VF test: \"o\ss ffl \char0\char0")
c.text(0, 9, r"\sffamily \fontsize{30}{35}\selectfont VF test: \"o\ss ffl \char0\char0")

# scaling test
unit.set(xscale=2)
t = c.text(0, 11, r"scale test", [color.rgb.green])
unit.set(xscale=1)
t = c.text(0, 11, r"scale test", [color.rgb.red])

# test font stripping (proper usedchar selection)
from pyx.dvi import mapfile
fontmap = mapfile.readfontmap(["download35.map"])
c.text(0, 12, r"usechar test (``fl'' should be typed):")
myrunner = text.texrunner()
myrunner.preamble(r"\font\pyxfont=phvr8t\pyxfont")
c.insert(myrunner.text(5.5, 12, r"\char'035", fontmap=fontmap))

myrunner2 = text.texrunner()
myrunner2.preamble(r"\font\pyxfont=ptmr8t\pyxfont")
c.insert(myrunner2.text(6.5, 12, r"\char'035", fontmap=fontmap))

# test for ligatures/kerning in virtual fonts
c.text(0, 13, r"usechar test (``affa'' should be typed):")
myrunner = text.texrunner()
myrunner.preamble(r"\font\pyxfont=ptmr7t\pyxfont")
c.insert(myrunner.text(6.0, 13, r"affa", fontmap=fontmap))

# test the specials
c.stroke(c.text(10, 2, r"Hello, \color{green}world!", [trafo.slant(1)]).path())
c.insert(c.text(10, 0, r"\begin{rotate}{90}\parbox{5cm}{rotated\\ in \LaTeX}\end{rotate}"))

d = canvas.canvas()
d.stroke(path.rect(0,0, 1,1))
d.stroke(path.line(0,0, 1,1))
d.stroke(path.line(1,0, 0,1))
d.writeEPSfile("sample")
c.insert(c.text(10, 0, r"""
    \textcolor{col0}{col0}
    \textcolor{col1}{col1}
    \textcolor{col2}{col2}
    \textcolor{col3}{col3}
    \textcolor{col4}{col4}
    \textcolor{col5}{col5}
    \textcolor{col6}{col6}
    \textcolor{col7}{col7}%
    """, [text.parbox(3.5)]))
c.insert(c.text(15, 0, r"""
    \colorbox{col1}{ColorBox}\\
    \fcolorbox{col5}{col6}{FColorBox}"""))

c.text(4, 2, r"{\color[cmyk]{0.1,0.2,0.3,0.4}c\color[gray]{0.5}o\color[hsb]{0.2,0.3,0.4}l\color[rgb]{0.2,0.4,0.6}o\color[RGB]{100,200,50}r}s!")

# curved text
sc = canvas.canvas(attrs=[trafo.mirror()])
p = path.path(path.moveto(-2, 0), path.curveto(-1, 0, -1, 1, 0, 1), path.curveto(1, 1, 1, 0, 2, 0))
sc.stroke(p, [deco.curvedtext(r"\PyX{} is fun!", textattrs=[trafo.mirror(), trafo.scale(0.5)]),
              deco.curvedtext("left", textattrs=[text.halign.left, text.vshift.mathaxis, trafo.mirror(), color.rgb.red], arclenfrombegin=0.5, exclude=0.1),
              deco.curvedtext(r"r{\color[rgb]{1,0,0}igh}t", textattrs=[text.halign.right, text.vshift.mathaxis, trafo.mirror(), trafo.scale(1.2)], arclenfromend=0.5, exclude=0.1)])
c.insert(sc, [trafo.translate(12, 12)])

# UnicodeEngine output with afm and pfm
ue_afm = text.UnicodeEngine()
c.insert(ue_afm.text(10, 9, "UnicodeEngine output (AFM)"))
ue_pfm = text.UnicodeEngine(font_metric_type=text.FontMetricType.pfm)
c.insert(ue_pfm.text(10, 8, "UnicodeEngine output (PFM)"))

c.writePDFfile("test_text", page_paperformat=document.paperformat.A4)
c.writeSVGfile("test_text", page_paperformat=document.paperformat.A4)

c.insert(c.text(10, 4, r"""%
    \fbox{\includegraphics[%
    %type=eps,             %% type of the file ... should not change anything --
                           %   BUG!!!!!!  size and filename information gets
                           %   wrong when this is used  ===> not supported!
    %command=...,          %% not supported!
    bb = 0 0 25 25,        %% bounding box in original size
    hiresbb=,              %! read high resolution in original file (if not bb)
    %viewport= 0 0 15 15,   %% bounding box with respect to bb
    %trim=1 1 1 1,          %% correction of the bounding box with respect to bb
    width=1in,             %! final width
    height=2in,            %! final height
    %totalheight=3in,       %% final height+depth
    %keepaspectratio=,      %! keep aspect ratio, but do not exceed width nor height
    angle=30,              %! wraps around include
    origin=tr,             %% one or two chars of 'lrtcbB' (B for baseline)
    %scale=2,               %! wraps around rotating and include
    %draft=,               %% do not print anything,
    clip=]%                %! directly in dvi
    {sample}}"""))
c.writeEPSfile("test_text", page_paperformat=document.paperformat.A4)



from pyx import *
c = canvas.canvas()

circle = path.circle(0, 0, 2)
line = path.line(-3, 1, 3, 2)

isects_circle, isects_line = circle.intersect(line)

arc1, arc2 = circle.split(isects_circle)

arc = arc1.arclen()<arc2.arclen() and arc1 or arc2

isects_line.sort()
line1, line2, line3 = line.split(isects_line)

segment = line2 << arc

c.fill(segment, [color.grey(0.5)])

c.stroke(circle, [style.linewidth.Thick])
c.stroke(line, [style.linewidth.Thick])

for isect in isects_circle:
    c.stroke(path.line(0, 0, *circle.at(isect)))

c.writePDFfile()



from pyx import *

g = graph.graphxy(width=8, x=graph.axis.linear(min=-2, max=2))
g.plot(graph.data.function("y(x)=exp(x)"))
g2 = g.insert(graph.graphxy(width=3.5, xpos=1, ypos=2,
                            x=graph.axis.linear(min=0, max=3),
                            y=graph.axis.linear(min=-2, max=2)))
g2.plot(graph.data.function("y(x)=log(x)"))
g.writeEPSfile()
g.writePDFfile()
g.writeSVGfile()



from math import *
from pyx import *
try:
    from math import radians, degrees
except ImportError:
    # fallback implementation for Python 2.1
    def radians(x): return x*pi/180
    def degrees(x): return x*180/pi

startbox = box.polygon(corners=[[0,-0.6], [0.5,0.1], [-0.25,0.1]])
endbox = box.polygon(corners=[[4.5,3.9], [5.5,4.0], [5.2,3.4]])

# the arc connector <<<
c1 = canvas.canvas()
for b in [startbox, endbox]:
    c1.stroke(b.path(), [style.linewidth.Thick, style.linejoin.round])
    c1.fill(path.circle_pt(b.center[0], b.center[1], 2))
absangle = degrees(atan2(endbox.center[1] - startbox.center[1], endbox.center[0] - startbox.center[0]))
relangle = 60
len = 2

# the direct connection
direct = path.line_pt(startbox.center[0], startbox.center[1], endbox.center[0], endbox.center[1])
c1.stroke(direct, [style.linestyle.dashed])

# the arc connector
l = connector.arc(startbox, endbox, relangle=relangle, boxdists=[0.0,0.0])
c1.stroke(l, [style.linewidth.Thick, color.rgb.red, deco.earrow.Large])

# the relangle parameter
comp1 = path.path(path.moveto(*direct.atbegin()),
                  path.rlineto(len*cos(radians(absangle + relangle)), len*sin(radians(absangle + relangle))))
c1.stroke(comp1)
ang = path.path(path.arc(direct.atbegin()[0], direct.atbegin()[1], 0.8*len, absangle, absangle+relangle))
c1.stroke(ang, [deco.earrow.large])
pos = ang.at(0.5*ang.arclen())
c1.text(pos[0], pos[1], r"~relangle", [text.halign.left])

# the bulge parameter
bulge = 0.5 * direct.arclen() * tan(0.5*radians(relangle))
bul = path.path(path.moveto(*direct.at(0.5*direct.arclen())),
                path.rlineto(bulge * cos(radians(absangle+90)), bulge * sin(radians(absangle+90))))
c1.stroke(bul, [deco.earrow.large])
pos = bul.at(0.5*bul.arclen())
c1.text(pos[0], pos[1], r"~(rel)bulge", [text.halign.left])

# >>>

# the curve connector <<<
c2 = canvas.canvas()
for b in [startbox, endbox]:
    c2.stroke(b.path(), [style.linewidth.Thick, style.linejoin.round])
    c2.fill(path.circle_pt(b.center[0], b.center[1], 2))
absangle = degrees(atan2(endbox.center[1] - startbox.center[1], endbox.center[0] - startbox.center[0]))
relangle1 = 60
relangle2 = 30
absbulge = 0
relbulge = 0.5
len = 2

# the direct connection
direct = path.line_pt(startbox.center[0], startbox.center[1], endbox.center[0], endbox.center[1])
c2.stroke(direct, [style.linestyle.dashed])

# the arc connector
l = connector.curve(startbox, endbox, relangle1=relangle1, relangle2=relangle2, absbulge=absbulge, relbulge=relbulge, boxdists=[0.0,0.0])
#l = connector.curve(startbox, endbox, absangle1=absangle+relangle1, absangle2=absangle+relangle2, absbulge=absbulge, relbulge=relbulge, boxdists=[0.0,0.0])
c2.stroke(l, [style.linewidth.Thick, color.rgb.red, deco.earrow.Large])

# the relangle parameters
# relangle1
c2.stroke(path.path(path.moveto(*direct.atbegin()),
                   path.rlineto(len*cos(radians(absangle + relangle1)),
                                len*sin(radians(absangle + relangle1)))))
ang = path.path(path.arc(direct.atbegin()[0], direct.atbegin()[1], 0.8*len, absangle, absangle+relangle1))
c2.stroke(ang, [deco.earrow.large])
pos = ang.at(0.5*ang.arclen())
c2.text(pos[0], pos[1], r"~relangle1", [text.halign.left])

# absangle1
c2.stroke(path.path(path.moveto(*direct.atbegin()), path.rlineto(len, 0)))
ang = path.path(path.arc(direct.atbegin()[0], direct.atbegin()[1], 0.5*len, 0, absangle+relangle1))
c2.stroke(ang, [deco.earrow.large])
pos = ang.at(0.2*ang.arclen())
c2.text(pos[0], pos[1], r"~absangle1", [text.halign.left])

# relangle2
c2.stroke(path.path(path.moveto(*direct.atend()),
                   path.rlineto(len*cos(radians(absangle)),
                                len*sin(radians(absangle)))))
c2.stroke(path.path(path.moveto(*direct.atend()),
                   path.rlineto(len*cos(radians(absangle + relangle2)),
                                len*sin(radians(absangle + relangle2)))))
ang = path.path(path.arc(direct.atend()[0], direct.atend()[1],
                         0.8*len, absangle, absangle+relangle2))
c2.stroke(ang, [deco.earrow.large])
pos = ang.at(0.5*ang.arclen())
c2.text(pos[0], pos[1], r"~relangle2", [text.halign.left])

# the bulge parameter
bulge = absbulge + direct.arclen() * relbulge
bul = path.path(path.moveto(*direct.atbegin()),
                path.rlineto(bulge * cos(radians(absangle+relangle1)), bulge * sin(radians(absangle+relangle1))))
c2.stroke(bul, [deco.earrow.large])
pos = bul.at(0.7*bul.arclen())
c2.text(pos[0], pos[1], r"~(rel)bulge", [text.halign.left])

bul = path.path(path.moveto(*direct.atend()),
                path.rlineto(-bulge * cos(radians(absangle+relangle2)), -bulge * sin(radians(absangle+relangle2))))
c2.stroke(bul, [deco.earrow.large])
pos = bul.at(0.7*bul.arclen())
c2.text(pos[0], pos[1], r"~(rel)bulge", [text.halign.left, text.vshift(1)])
# >>>


# write everything
c1.insert(c2, [trafo.translate(6.5, 0)])
c1.writePDFfile()

# vim:foldmethod=marker:foldmarker=<<<,>>>



import math
from pyx import *

class phasecolorsurface(graph.style.surface):

    def midcolor(self, *c):
        return math.atan2(sum([math.sin(x) for x in c]), sum([math.cos(x) for x in c]))

g = graph.graphxyz(size=3.5, zscale=0.3, projector=graph.graphxyz.central(3, 25, 8),
                   x=graph.axis.lin(painter=None),
                   y=graph.axis.lin(painter=None),
                   z=graph.axis.lin(painter=None),
                   x2=None, y2=None)
g.plot(graph.data.file("example.dat", x=1, y=2, z=3, color=4), [phasecolorsurface(gradient=color.rgbgradient.Rainbow, coloraxis=graph.axis.lin(min=-math.pi, max=math.pi), backcolor=color.rgb.black, keygraph=None)])
g.writeEPSfile("example")



# this example is adapted from a MetaPost example by Hans Hagen

import sys; sys.path.insert(0, "../..")
from pyx import *
from solve import scalar, vector, solver

# define the inner quadrilateral
p = vector([0, 0])
q = vector([2, 5])
r = vector([8, 6])
s = vector([5, -1])

# some points of the outer squares are equal to points of the inner quadrilateral
z11 = p
z12 = q
z21 = q
z22 = r
z31 = r
z32 = s
z41 = s
z42 = p

# complete the definition of the outer squares
z1diff = z12 - z11
z2diff = z22 - z21
z3diff = z32 - z31
z4diff = z42 - z41

z13 = vector([z12.x-z1diff.y, z12.y+z1diff.x])
z14 = vector([z11.x-z1diff.y, z11.y+z1diff.x])
z23 = vector([z22.x-z2diff.y, z22.y+z2diff.x])
z24 = vector([z21.x-z2diff.y, z21.y+z2diff.x])
z33 = vector([z32.x-z3diff.y, z32.y+z3diff.x])
z34 = vector([z31.x-z3diff.y, z31.y+z3diff.x])
z43 = vector([z42.x-z4diff.y, z42.y+z4diff.x])
z44 = vector([z41.x-z4diff.y, z41.y+z4diff.x])

# define the centers of the outer squares
z1 = 0.5*(z11 + z13)
z2 = 0.5*(z21 + z23)
z3 = 0.5*(z31 + z33)
z4 = 0.5*(z41 + z43)

# define the crossing point by some equations
z0 = vector(2)
solver.eq(z0, z1 + scalar()*(z3-z1))
solver.eq(z0, z2 + scalar()*(z4-z2))

# finally draw the result

def line(p1, p2):
    return path.line(float(p1.x), float(p1.y), float(p2.x), float(p2.y))

def square(p1, p2, p3, p4):
    return path.path(path.moveto(float(p1.x), float(p1.y)),
                     path.lineto(float(p2.x), float(p2.y)),
                     path.lineto(float(p3.x), float(p3.y)),
                     path.lineto(float(p4.x), float(p4.y)),
                     path.closepath())

c = canvas.canvas()

c.stroke(square(z11, z12, z13, z14))
c.stroke(square(z21, z22, z23, z24))
c.stroke(square(z31, z32, z33, z34))
c.stroke(square(z41, z42, z43, z44))

c.stroke(line(z11, z13), [style.linestyle.dashed, style.linewidth.Thin])
c.stroke(line(z12, z14), [style.linestyle.dashed, style.linewidth.Thin])
c.stroke(line(z21, z23), [style.linestyle.dashed, style.linewidth.Thin])
c.stroke(line(z22, z24), [style.linestyle.dashed, style.linewidth.Thin])
c.stroke(line(z31, z33), [style.linestyle.dashed, style.linewidth.Thin])
c.stroke(line(z32, z34), [style.linestyle.dashed, style.linewidth.Thin])
c.stroke(line(z41, z43), [style.linestyle.dashed, style.linewidth.Thin])
c.stroke(line(z42, z44), [style.linestyle.dashed, style.linewidth.Thin])

c.stroke(square(p, q, r, s), [color.rgb.red, style.linewidth.THick])

c.stroke(line(z1, z3), [color.rgb.green])
c.stroke(line(z2, z4), [color.rgb.green])

c.fill(path.circle(float(z0.x), float(z0.y), 0.1), [color.rgb.blue])

c.writeEPSfile()
c.writePDFfile()




from pyx import *
c = canvas.canvas()
p = path.curve(0, 0, 0.05, 0.3, 0.2, 0.5, 0.5, 0.5)
d = [deco.filled([color.gray(0.7)]),
     deco.stroked([color.gray(0.3), style.linejoin.round])]
a = deco.earrow.large(d)
c.stroke(p, [style.linestyle.dashed, a])
c.writeEPSfile("arrow")



import re
from pyx import *

text.set(text.LatexRunner)
text.preamble(r"\renewcommand{\familydefault}{\ttdefault}")
c = canvas.canvas()

# positioning is quite ugly ... but it works at the moment
x = 0
y = 0
dx = 4.8
dy = -0.65
lastmodel = 0

# we could use (gray|rgb|cmyk).__dict__ to get the instances, but we
# would loose the ordering ... instead we just parse the file:

# TODO: code something along the lines of c.l.p post cbs38g$2kp$04$1@news.t-online.com

p = re.compile("(?P<id>(?P<model>[a-z]+)\\.(?P<name>[a-z]+)) += (?P=model)\\([0-9\\., ]+\\)\n", re.IGNORECASE)
with open(color.__file__) as f:
    for line in f:
        m = p.match(line)
        if m:
            if lastmodel and (m.group("model") != lastmodel):
               y += dy
            myc = color.__dict__[m.group("model")].__dict__[m.group("name")]
            c.stroke(path.line(x + 0.1, y + 0.1, x + 0.4, y + 0.4), [myc])
            c.stroke(path.line(x + 0.4, y + 0.1, x + 0.1, y + 0.4), [myc])
            c.fill(path.rect(x + 0.5, y, 1, 0.5), [myc])
            c.stroke(path.line(x + 0.6, y + 0.1, x + 0.9, y + 0.4), [color.gray.black])
            c.stroke(path.line(x + 0.9, y + 0.1, x + 0.6, y + 0.4), [color.gray.black])
            c.stroke(path.line(x + 1.1, y + 0.1, x + 1.4, y + 0.4), [color.gray.white])
            c.stroke(path.line(x + 1.4, y + 0.1, x + 1.1, y + 0.4), [color.gray.white])
            c.text(x + 1.7, y + 0.15, m.group("id"), [text.size.footnotesize])
            y += dy
            lastmodel = m.group("model")
            if y < -16.5:
                y = 0
                x += dx

c.writePDFfile()



import sys
if sys.path[0] != "../..":
    sys.path.insert(0, "../..")

import unittest

from pyx import mathutils

class PolynomTestCase(unittest.TestCase):

    def makePolyRoots(self, *rs):
        # creates a polynom with zeros rs
        if len(rs) == 1:
            return [1, -rs[0]]
        cs = self.makePolyRoots(*rs[1:])
        return [cim1-rs[0]*ci for cim1, ci in zip(cs+[0], [0]+cs)]

    def makePolyNonroots(self, cs, a, b):
        # adds a factor ((x-a)**2 + b) to the polynom with coefficients cs
        # which can be used (for b > 0) to construct polynoms with complex roots
        return [cim2-2*a*cim1+(a*a+b)*ci for cim2, cim1, ci in zip(cs+[0, 0], [0]+cs+[0], [0, 0]+cs)]

    def compareRoots(self, found, should):
        found.sort()
        # we remove degeneracies in found, since we do *not* claim to properly handle degeneracies
        i = 1
        while len(found) > i:
            if found[i] - found[i-1] < 1e-7:
                del found[i]
            else:
                i += 1
        self.assertEqual(len(found), len(should))
        for r1, r2 in zip(found, should):
            self.assertAlmostEqual(r1, r2)

    def testConstant(self):
        self.compareRoots(mathutils.realpolyroots(1), [])
        self.compareRoots(mathutils.realpolyroots(0), [0])

    def testLinear(self):
        self.compareRoots(mathutils.realpolyroots(0, 1), [])
        self.compareRoots(mathutils.realpolyroots(0, 0), [0])
        self.compareRoots(mathutils.realpolyroots(1, 1), [-1])
        self.compareRoots(mathutils.realpolyroots(2, 0), [0])
        self.compareRoots(mathutils.realpolyroots(4, -1), [0.25])
        self.compareRoots(mathutils.realpolyroots(*self.makePolyRoots(1.7)), [1.7])
        self.compareRoots(mathutils.realpolyroots(*self.makePolyRoots(-42)), [-42])

    def testQuadratic(self):
        self.compareRoots(mathutils.realpolyroots(0, 1, 1), [-1])
        self.compareRoots(mathutils.realpolyroots(1, -4, 3), [1, 3])
        self.compareRoots(mathutils.realpolyroots(*self.makePolyRoots(1.7, 2.3)), [1.7, 2.3])
        self.compareRoots(mathutils.realpolyroots(*self.makePolyRoots(2.3, 1.7)), [1.7, 2.3])
        self.compareRoots(mathutils.realpolyroots(*self.makePolyRoots(1.7, -2.3)), [-2.3, 1.7])
        self.compareRoots(mathutils.realpolyroots(*self.makePolyRoots(-2.3, 1.7)), [-2.3, 1.7])
        self.compareRoots(mathutils.realpolyroots(*self.makePolyRoots(1.7, 1.7)), [1.7])
        self.compareRoots(mathutils.realpolyroots(*self.makePolyNonroots([1], 1.7, 1)), [])
        self.compareRoots(mathutils.realpolyroots(*self.makePolyNonroots([1], 1.7, -1)), [0.7, 2.7])
        self.compareRoots(mathutils.realpolyroots(*self.makePolyNonroots([1], 1.7, -4)), [-0.3, 3.7])

    def testCubic(self):
        self.compareRoots(mathutils.realpolyroots(0, 0, 0, 1), [])
        self.compareRoots(mathutils.realpolyroots(0, 0, 1, 1), [-1])
        self.compareRoots(mathutils.realpolyroots(0, 1, -4, 3), [1, 3])
        self.compareRoots(mathutils.realpolyroots(1, -9, 23, -15), [1, 3, 5])
        self.compareRoots(mathutils.realpolyroots(*self.makePolyRoots(1.7, 2.3, 4.2)), [1.7, 2.3, 4.2])
        self.compareRoots(mathutils.realpolyroots(*self.makePolyRoots(1.7, -2.3, 4.2)), [-2.3, 1.7, 4.2])
        self.compareRoots(mathutils.realpolyroots(*self.makePolyRoots(1.7, -2.3, -4.2)), [-4.2, -2.3, 1.7])
        self.compareRoots(mathutils.realpolyroots(*self.makePolyNonroots(self.makePolyRoots(1.7), 2.3, 1)), [1.7])
        self.compareRoots(mathutils.realpolyroots(*self.makePolyNonroots(self.makePolyRoots(1.7), 2.3, -1)), [1.3, 1.7, 3.3])
        self.compareRoots(mathutils.realpolyroots(*self.makePolyNonroots(self.makePolyRoots(1.7), 2.3, -4)), [0.3, 1.7, 4.3])

    def testQuartic(self):
        self.compareRoots(mathutils.realpolyroots(0, 0, 0, 0, 1), [])
        self.compareRoots(mathutils.realpolyroots(0, 0, 0, 1, 1), [-1])
        self.compareRoots(mathutils.realpolyroots(0, 1, -9, 23, -15), [1, 3, 5])
        self.compareRoots(mathutils.realpolyroots(1, -16, 86, -176, 105), [1, 3, 5, 7])
        self.compareRoots(mathutils.realpolyroots(*self.makePolyRoots(1.7, 2.3, 4.2, 13)), [1.7, 2.3, 4.2, 13])
        self.compareRoots(mathutils.realpolyroots(*self.makePolyRoots(1.7, -2.3, 4.2, 13)), [-2.3, 1.7, 4.2, 13])
        self.compareRoots(mathutils.realpolyroots(*self.makePolyRoots(1.7, -2.3, -4.2, 13)), [-4.2, -2.3, 1.7, 13])
        self.compareRoots(mathutils.realpolyroots(*self.makePolyNonroots(self.makePolyRoots(1.7, 2.3), 4.2, 1)), [1.7, 2.3])
        self.compareRoots(mathutils.realpolyroots(*self.makePolyNonroots(self.makePolyRoots(1.7, 2.3), 4.2, -1)), [1.7, 2.3, 3.2, 5.2])
        self.compareRoots(mathutils.realpolyroots(*self.makePolyNonroots(self.makePolyRoots(1.7, 2.3), 4.2, -4)), [1.7, 2.2, 2.3, 6.2])
        self.compareRoots(mathutils.realpolyroots(*self.makePolyNonroots(self.makePolyRoots(1.7, -2.3), 4.2, -1)), [-2.3, 1.7, 3.2, 5.2])
        self.compareRoots(mathutils.realpolyroots(*self.makePolyNonroots(self.makePolyRoots(1.7, -2.3), 4.2, -4)), [-2.3, 1.7, 2.2, 6.2])
        self.compareRoots(mathutils.realpolyroots(*self.makePolyNonroots(self.makePolyNonroots([1], 1.7, 1), 2.3, 1)), [])
        self.compareRoots(mathutils.realpolyroots(*self.makePolyNonroots(self.makePolyNonroots([1], 1.7, 1), 2.3, -1)), [1.3, 3.3])


if __name__ == "__main__":
    unittest.main()



from math import sin, cos, pi
from pyx import *

r = 1.5

# create a box list of regular polygons
boxes = [box.polygon([(-r*sin(i*2*pi/n), r*cos(i*2*pi/n))
                      for i in range(n)])
         for n in range(3, 8)]
# tile with spacing 0 horizontally
box.tile(boxes, 0, 1, 0)

c = canvas.canvas()
for b in boxes:
    # plot the boxes path
    c.stroke(b.path(), [color.rgb.green])
    # a second time with bezier rounded corners
    c.stroke(b.path(), [deformer.smoothed(radius=0.5), color.rgb.red])
c.writeEPSfile()
c.writePDFfile()
c.writeSVGfile()



from pyx import *

text.set(text.LatexRunner)
text.preamble(r"\renewcommand{\familydefault}{\ttdefault}")
c = canvas.canvas()

# positioning is quite ugly ... but it works at the moment
x = 0
y = 0
dx = 6
dy = -0.65
length = 1.2

def drawdeco(name, showpath=0, default=0):
    global x,y
    p = path.path(path.moveto(x + 0.1, y+0.1 ),
                       path.rlineto(length/2.0, 0.3),
                       path.rlineto(length/2.0, -0.3))
    c.stroke(p, [style.linewidth.THIck,  eval("deco."+name)])
    if showpath:
        c.stroke(p, [style.linewidth.Thin, color.gray.white])
    if default:
        name = name + r"\rm\quad (default)"
    c.text(x + 1.5, y + 0.15, name, [text.size.footnotesize])
    y += dy
    if y < -16:
        y = 0
        x += dx

drawdeco("earrow.Small")
drawdeco("earrow.small")
drawdeco("earrow.normal")
drawdeco("earrow.large")
drawdeco("earrow.Large")

y += dy

drawdeco("barrow.normal")

y += dy

drawdeco("earrow.Large([deco.filled([color.rgb.red]), style.linewidth.normal])")
drawdeco("earrow.normal(constriction=None)")
drawdeco("earrow.Large([style.linejoin.round])")
drawdeco("earrow.Large([deco.stroked.clear])")

c.writePDFfile()



# Sierpinski triangle
# contributed by Gerhard Schmid

from math import sqrt
from pyx import *

# triangle geometry
l = 10
h = 0.5 * sqrt(3) * l

# base triangle path
p = path.path(path.moveto(0, 0),
              path.lineto(l, 0),
              path.lineto(0.5 * l, h),
              path.closepath())

for i in range(6):
    # path is scaled down ...
    p = p.transformed(trafo.scale(0.5))
    # ... and three times plotted (translated accordingly)
    p += ( p.transformed(trafo.translate(0.5 * l, 0)) +
           p.transformed(trafo.translate(0.25 * l, 0.5 * h)))

c = canvas.canvas()
c.stroke(p, [style.linewidth.Thin])
c.writeEPSfile()
c.writePDFfile()
c.writeSVGfile()



from pyx import *

c = canvas.canvas()

g1 = c.insert(graph.graphxy(width=8,
                            x=graph.axis.linear(min=0, max=1),
                            x2=graph.axis.linkedaxis()))

g2 = c.insert(graph.graphxy(width=8, ypos=g1.height+0.5,
                            x=graph.axis.linkedaxis(),
                            x2=graph.axis.log(min=1, max=1000)))

g1.axes["x2"].setlinkedaxis(g2.axes["x2"])
g2.axes["x"].setlinkedaxis(g1.axes["x"])

g1.plot(graph.data.function("y(x)=2*exp(-30*x)-exp(-3*x)"))
g2.plot(graph.data.function("y(x)=cos(20*x)*exp(-2*x)"))

c.writeEPSfile()
c.writePDFfile()
c.writeSVGfile()



import sys
if sys.path[0] != "../..":
    sys.path.insert(0, "../..")

import unittest

import io
from pyx.graph import data

class DataTestCase(unittest.TestCase):

    def testPoints(self):
        mydata = data.points([[1, 2, 3], [4, 5, 6]], a=1, b=2)
        self.assertEqual(mydata.columndata[0], [1, 2])
        self.assertEqual(mydata.columns["a"], [1, 4])
        self.assertEqual(mydata.columndata[2], [2, 5])
        self.assertEqual("c" in list(mydata.columns.keys()), 0)

    def testValues(self):
        mydata = data.values(a=[1, 4])
        self.assertEqual(mydata.columns["a"], [1, 4])
        self.assertEqual("c" in list(mydata.columns.keys()), 0)

    def testData(self):
        mydata = data.points([[1], [2]], a=1)
        mydata2 = data.data(mydata, a="2*a", b="2*$1*a", c="4*$(i)*a*$(-1)", context={"i":1})
        self.assertEqual(mydata.columns["a"], [1, 2])
        self.assertAlmostEqual(mydata2.columns["a"][0], 2.0)
        self.assertAlmostEqual(mydata2.columns["a"][1], 4.0)
        self.assertAlmostEqual(mydata2.columns["b"][0], 2.0)
        self.assertAlmostEqual(mydata2.columns["b"][1], 8.0)
        self.assertAlmostEqual(mydata2.columns["c"][0], 4.0)
        self.assertAlmostEqual(mydata2.columns["c"][1], 32.0)
        mydata3 = data.data(mydata2, a="b", b="2*c")
        self.assertEqual(mydata3.columns["a"], mydata2.columns["b"])
        self.assertAlmostEqual(mydata3.columns["b"][0], 2*mydata2.columns["c"][0])
        self.assertAlmostEqual(mydata3.columns["b"][1], 2*mydata2.columns["c"][1])

        a = "nothing"
        two = 2
        f = lambda x: x*x
        mydata = data.points([[1], [2]], a=1)
        mydata2 = data.data(mydata, b="two*a", c="two*$1*a", d="f($1)", context=locals())
        self.assertEqual(mydata.columndata[0], [1, 2])
        self.assertAlmostEqual(mydata2.columns["b"][0], 2.0)
        self.assertAlmostEqual(mydata2.columns["b"][1], 4.0)
        self.assertAlmostEqual(mydata2.columns["c"][0], 2.0)
        self.assertAlmostEqual(mydata2.columns["c"][1], 8.0)
        self.assertAlmostEqual(mydata2.columns["d"][0], 1.0)
        self.assertAlmostEqual(mydata2.columns["d"][1], 4.0)

    def testFile(self):
        testfile = io.StringIO("""#a
0
1 eins
2 "2"
3 x"x""")
        mydata = data.file(testfile, row=0, a="a", b=2)
        self.assertEqual(mydata.columns["row"], [1, 2, 3, 4])
        self.assertAlmostEqual(mydata.columns["a"][0], 0.0)
        self.assertAlmostEqual(mydata.columns["a"][1], 1.0)
        self.assertAlmostEqual(mydata.columns["a"][2], 2.0)
        self.assertAlmostEqual(mydata.columns["a"][3], 3.0)
        self.assertEqual(mydata.columns["b"][0], None)
        self.assertEqual(mydata.columns["b"][1], "eins")
        self.assertEqual(mydata.columns["b"][2], "2")
        self.assertEqual(mydata.columns["b"][3], "x\"x")
        testfile = io.StringIO("""#a
0
1
2
3
4
5
6
7
8
9""")
        mydata = data.file(testfile, title="title", skiphead=3, skiptail=2, every=2, row=0)
        self.assertEqual(mydata.columns["row"], [4, 6, 8])
        self.assertEqual(mydata.title, "title")

    def testSec(self):
        testfile = io.StringIO("""[sec1]
opt1=a1
opt2=a2
val=1
val=2

[sec2]
opt1=a4
opt2=a5
val=2
val=1

[sec1]
opt3=a3""")
        mydata = data.conffile(testfile, sec=0, a="opt1", b="opt2", c="opt3", d="val")
        self.assertEqual(mydata.columns["sec"], ["sec1", "sec2"])
        self.assertEqual(mydata.columns["a"], ["a1", "a4"])
        self.assertEqual(mydata.columns["b"], ["a2", "a5"])
        self.assertEqual(mydata.columns["c"], ["a3", None])
        self.assertAlmostEqual(mydata.columns["d"][0], 2.0)
        self.assertAlmostEqual(mydata.columns["d"][1], 1.0)

    def testParamfunction(self):
        mydata = data.paramfunction("k", 0, 9, "x, y = k, -k", points=10)
        for i in range(10):
            self.assertEqual(mydata.columns["x"][i], i)
            self.assertEqual(mydata.columns["y"][i], -i)


if __name__ == "__main__":
    unittest.main()



import sys, os, os.path, html, io, codecs, glob, re, warnings
import keyword, token, tokenize
import xml.dom.minidom
from zope.pagetemplate.pagetemplate import PageTemplate
from PIL import Image

sys.path[:0]=[".."]
import pyx

def PageTemplateFile(filename):
    pt = PageTemplate()
    pt.write(codecs.open(filename, "r", encoding="utf-8").read())
    return pt

_KEYWORD = token.NT_OFFSET

tokclasses = {token.NUMBER: 'number',
              token.OP: 'op',
              token.STRING: 'string',
              tokenize.COMMENT: 'comment',
              token.NAME: 'name',
              _KEYWORD: 'keyword'}

class MakeHtml:

    def fromPython(self, input):
        input = io.StringIO(input)
        self.output = io.StringIO()
        self.col = 0
        self.tokclass = None
        self.output.write("<pre id=python>")
        for token in tokenize.generate_tokens(input.readline):
            self.tokeneater(*token)
        if self.tokclass is not None:
            self.output.write('</span>')
        self.output.write("</pre>\n")
        return self.output.getvalue()

    def tokeneater(self, toktype, toktext, xxx_todo_changeme, xxx_todo_changeme1, line):
        (srow, scol) = xxx_todo_changeme
        (erow, ecol) = xxx_todo_changeme1
        if toktype == token.ERRORTOKEN:
            raise RuntimeError("ErrorToken occured")
        if toktype in [token.NEWLINE, tokenize.NL]:
            self.output.write('\n')
            self.col = 0
        else:
            # map token type to a color group
            if token.LPAR <= toktype and toktype <= token.OP:
                toktype = token.OP
            elif toktype == token.NAME and keyword.iskeyword(toktext):
                toktype = _KEYWORD

            # restore whitespace
            assert scol >= self.col
            self.output.write(" "*(scol-self.col))

            try:
                tokclass = tokclasses[toktype]
            except KeyError:
                tokclass = None
            if self.tokclass is not None and tokclass != self.tokclass:
                self.output.write('</span>')
            if tokclass is not None and tokclass != self.tokclass:
                self.output.write('<span class="%s">' % tokclass)
            self.output.write(html.escape(toktext))
            self.tokclass = tokclass

            # calculate new column position
            self.col = scol + len(toktext)
            newline = toktext.rfind("\n")
            if newline != -1:
                self.col = len(toktext) - newline - 1

    emptypattern = re.compile(r"\s*$")
    parpattern = re.compile(r"([ \t]*\n)*(?P<data>(\S[^\n]*\n)+)(([ \t]*\n)+.*)?")
    parmakeline = re.compile(r"\s*[\r\n]\s*")
    parmakeinline = re.compile(r"`(.*?)`")
    parmakebold = re.compile(r"'''(.*?)'''")
    parmakeitalic = re.compile(r"''(.*?)''")
    parmakeref = re.compile(r"\[([^]]*)\s([^\s]*)\]")
    codepattern = re.compile(r"([ \t]*\n)*(?P<data>(([ \t]+[^\n]*)?\n)+)")
    indentpattern = re.compile(r"([ \t]*\n)*(?P<indent>[ \t]+)")

    def fromText(self, input, bend=""):
        title = None
        pos = 0
        output = io.StringIO()
        while not self.emptypattern.match(input, pos):
            par = self.parpattern.match(input, pos)
            if par:
                pos = par.end("data")
                par = par.group("data").strip()
                par = par.replace("__version__", pyx.__version__)
                par = par.replace("&", "&amp;")
                par = par.replace("<", "&lt;")
                par = par.replace(">", "&gt;")
                par = par.replace("\"", "&quot;")
                par = self.parmakeline.subn(" ", par)[0]
                par = self.parmakeinline.subn(r"<code>\1</code>", par)[0]
                par = self.parmakebold.subn(r"<strong>\1</strong>", par)[0]
                par = self.parmakeitalic.subn(r"<em>\1</em>", par)[0]
                par = self.parmakeref.subn(r'<a href="\2">\1</a>', par)[0]
                if not title:
                    title = par
                else:
                    bends = 0
                    while par.startswith("!"):
                        if not bend:
                            warnings.warn("ignore bend sign")
                            break
                        bends += 1
                        par = par[1:]
                    output.write("%s<p>%s</p>\n" % (bend*bends, par))
            else:
                code = self.codepattern.match(input, pos)
                if not code:
                    raise RuntimeError("couldn't parse text file")
                pos = code.end("data")
                code = code.group("data")
                indent = self.indentpattern.match(code).group("indent")
                code = re.subn(r"\s*[\r\n]%s" % indent, "\n", code.strip())[0]
                if len(indent.expandtabs()) >= 4:
                    code = self.fromPython(code + "\n")
                else:
                    code = "<pre>%s</pre>" % code
                output.write("<div class=\"codeindent\">%s</div>\n" % code)
        text = output.getvalue()
        shorttext = text.split("...")[0]
        text = text.replace("...", "", 1)
        return title, shorttext, text

makehtml = MakeHtml()


class example:

    def __init__(self, basesrcdir, dir, basename):
        self.basename = basename
        if dir:
            name = os.path.join(dir, basename)
        else:
            name = basename
        relname = os.path.join(basesrcdir, name)
        self.filename = "%s.py" % name
        self.code = makehtml.fromPython(codecs.open("%s.py" % relname, encoding="utf-8").read())
        self.html = "%s.html" % basename
        self.png = "%s.png" % basename
        self.width, self.height = Image.open("%s.png" % relname).size
        self.thumbpng = "%s_thumb.png" % basename
        self.thumbwidth, self.thumbheight = Image.open("%s_thumb.png" % relname).size
        self.downloads = []
        for suffix in ["py", "ipynb", "dat", "jpg", "eps", "pdf", "svg"]:
            try:
                filesize = "%.1f KB" % (os.path.getsize("%s.%s" % (relname, suffix)) / 1024.0)
            except OSError:
                pass
            else:
                self.downloads.append({"filename": "%s.%s" % (basename, suffix),
                                       "suffixname": ".%s" % suffix,
                                       "filesize": filesize,
                                       "iconname": "%s.png" % suffix})
        if os.path.exists("%s.txt" % relname):
            self.title, self.shorttext, self.text = makehtml.fromText(codecs.open("%s.txt" % relname, encoding="utf-8").read(), bend="<div class=\"examplebend\"><img src=\"../../bend.png\" width=22 height=31></div>\n")
        else:
            self.title = basename
            self.shorttext = self.text = None


def mkrellink(linkname, options):
    # returns a string containing the relative url for linkname (an absolute url)
    pagename = options["pagename"]
    while linkname.find("/") != -1 and pagename.find("/") != -1:
        linknamefirst, linknameother = linkname.split("/", 1)
        pagenamefirst, pagenameother = pagename.split("/", 1)
        if linknamefirst == pagenamefirst:
            linkname = linknameother
            pagename = pagenameother
        else:
            break
    for i in pagename.split("/")[:-1]:
        linkname = "../" + linkname
    return linkname

maintemplate = PageTemplateFile("maintemplate.pt")

latestnews = 2
newsdom = xml.dom.minidom.parse("news.pt")
news = "".join(["%s%s" % (dt.toxml(), dd.toxml())
                for dt, dd in zip(newsdom.getElementsByTagName("dt")[:latestnews],
                                  newsdom.getElementsByTagName("dd")[:latestnews])])

for ptname in glob.glob("*.pt"):
    if ptname not in ["maintemplate.pt", "exampleindex.pt", "examples.pt", "example.pt"]:
        htmlname = "%s.html" % ptname[:-3]
        template = PageTemplateFile(ptname)
        content = template(pagename=htmlname,
                           maintemplate=maintemplate,
                           mkrellink=mkrellink,
                           version=pyx.__version__,
                           news=news)
        codecs.open("build/%s" % htmlname, "w", encoding="utf-8").write(content)


def processexamples(basedir, gallery=False):
    exampleindextemplate = PageTemplateFile("exampleindex.pt")
    examplestemplate = PageTemplateFile("examples.pt")
    exampletemplate = PageTemplateFile("example.pt")

    exampledirs = [None]
    examplepages = []
    for dir in open(os.path.join("..", basedir, "INDEX")).readlines():
        dir = dir.strip()
        if dir.endswith("/"):
            exampledirs.append(dir)
            dir = dir.rstrip("/")
            try:
                title = open(os.path.join("..", basedir, dir, "README")).readline().strip()
            except IOError:
                title = dir
            examplepages.append({"dir": dir, "title": title})

    prev = None
    for dirindex, dir in enumerate(exampledirs):
        if dir:
            srcdir = os.path.join("..", basedir, dir)
            destdir = os.path.join(basedir, dir)
            bend = "<div class=\"examplebend\"><img src=\"../../bend.png\" width=22 height=31></div>\n"
        else:
            srcdir = os.path.join("..", basedir)
            destdir = basedir
            bend = "<div class=\"examplebend\"><img src=\"../bend.png\" width=22 height=31></div>\n"
        try:
            nextdir = exampledirs[dirindex + 1]
            nextdir = os.path.join(basedir, nextdir)
        except IndexError:
            nextdir = None
        try:
            title, shorttext, text = makehtml.fromText(open(os.path.join(srcdir, "README")).read(), bend=bend)
        except IOError:
            title = dir
            text = ""
        examples = [example(os.path.join("..", basedir), dir, item.strip())
                    for item in open(os.path.join(srcdir, "INDEX")).readlines()
                    if item[-2] != "/"]
        htmlname = os.path.join(destdir, "index.html")
        if dir:
            template = examplestemplate
            next = os.path.join(destdir, examples[0].html)
        else:
            template = exampleindextemplate
            next = os.path.join(nextdir, "index.html")
        content = template(pagename=htmlname,
                           maintemplate=maintemplate,
                           version=pyx.__version__,
                           dir=dir,
                           title=title,
                           text=text,
                           examples=examples,
                           subpages=examplepages,
                           mkrellink=mkrellink,
                           prev=prev,
                           next=next,
                           gallery=gallery)
        codecs.open("build/%s" % htmlname, "w", encoding="utf-8").write(content)
        prev = os.path.join(destdir, "index.html")
        if dir:
            for exampleindex, aexample in enumerate(examples):
                try:
                    next = os.path.join(destdir, examples[exampleindex+1].html)
                except (TypeError, IndexError):
                    if nextdir:
                        next = os.path.join(nextdir, "index.html")
                    else:
                        next = None
                htmlname = os.path.join(destdir, "%s.html" % aexample.basename)
                content = exampletemplate(pagename=htmlname,
                                          maintemplate=maintemplate,
                                          version=pyx.__version__,
                                          dir=dir,
                                          example=aexample,
                                          subpages=examplepages,
                                          mkrellink=mkrellink,
                                          prev=prev,
                                          next=next,
                                          gallery=gallery)
                codecs.open("build/%s" % htmlname, "w", encoding="utf-8").write(content)
                prev = os.path.join(destdir, aexample.html)


processexamples("examples")
processexamples("gallery", gallery=True)



#!/usr/bin/env python
import sys
if sys.path[0] != "../..":
    sys.path.insert(0, "../..")

import unittest

import glob

def regressionTest():
    names = [name[:-3] for name in glob.glob("test_*.py")]
    modules = list(map(__import__, names))
    load = unittest.defaultTestLoader.loadTestsFromModule
    return unittest.TestSuite(list(map(load, modules)))

if __name__ == "__main__":
    unittest.main(defaultTest="regressionTest")




#!/usr/bin/python3
import sys, math
import numpy
sys.path.insert(0, "../..")
import pyx
from pyx import *
from pyx.graph import axis

c = canvas.canvas()

def plot_xy_function(xmin=0.72, xmax=7.85):

    xaxis = graph.axis.linear(title=r"$x$", min=xmin, max=xmax)

    yaxis = graph.axis.log(title=r"$y$", min=None, max=None) #, parter=yparter, rater=yrater)

    g = graph.graphxy(width=10, x=xaxis, y=yaxis, key=graph.key.key())
    g.plot(graph.data.function("y(x) = -x**2", title=r"$-x^2$"))
    return g

def plot_color():
    # the example function to plot:
    def fct(x, y):
        return (3*x**5*y - 10*x**3*y**3 + 3*x*y**5) / 16.0

    zero_cutoff = 1e-5

    # create data on a grid:
    datapts = []
    for i, x in enumerate(numpy.linspace(-1.5, 1.5, num=70, endpoint=True)):
        for j, y in enumerate(numpy.linspace(-1.5, 1.5, num=70, endpoint=True)):
            f = fct(x, y)
            # Skip data too close to zero:
            if f > zero_cutoff:
                datapts.append((x, y, ('pos', f)))
            elif f < -zero_cutoff:
                datapts.append((x, y, ('neg', f)))

    # plot:
    width, height = 10, 10

    # define color gradients:
    gradientRedWhiteBlue = pyx.color.functiongradient_rgb(
        f_r=lambda x: 1-0.5*(1-2*x)**2 if x < 0.5 else 2 - 2*x,
        f_g=lambda x: 2*x if x < 0.5 else 2 - 2*x,
        f_b=lambda x: 2*x if x < 0.5 else 1 - 2*(x-0.5)**2)


    # plot:
    g = pyx.graph.graphxy(width=width, height=height,
        x=pyx.graph.axis.linear(title=r"$x$"),
        y=pyx.graph.axis.linear(title=r"$y$"),
    )
    kg = graph.graphx(xpos=width+0.5, length=10, x=graph.axis.split(dist=0.1, subaxes={"neg": graph.axis.log(rater_density=2),
                                                                                       "pos": graph.axis.log(rater_density=2, texter_basetexter_prefix='+')}))
    g.plot(pyx.graph.data.points(datapts, x=1, y=2, color=3, title=None), [graph.style.density(gradient=gradientRedWhiteBlue, keygraph=kg)])
    g.insert(kg)

    return g


c = canvas.canvas()
c.insert(plot_xy_function())
c.insert(plot_color(), [trafo.translate(0, -15)])
c.writePDFfile()




#!/usr/bin/env python
import sys;sys.path.insert(0, "..")
from math import *
#sys.path.insert(0, os.path.expanduser("~/python/pyx-trunk"))
from pyx import *
# File for invalid parametrisations of Bezier curves
# Some examples

def Dx_m(Dy):
    return (-1-2*Dy-sqrt((1+2*Dy)**2+3))/3.0

def Dx_p(Dy):
    return (-1-2*Dy+sqrt((1+2*Dy)**2+3))/3.0

def pass00(Dy):
    return 1.0/(3.0*(Dy+1))

def pass10(Dy):
    return (1.0+Dy**3)/(3.0*(Dy+1))

def acurve(Dx, Dy, trafo_reverse=False):
    p = path.curve(0,0, 0,1, Dx,1+Dy, 1,0)
    if trafo_reverse:
        # calculate the trafo to put p2=(1,-1):
        # (for the right branch only)
        x2, y2 = Dx, Dy+1
        tr1 = trafo.scale(1, -1.0/y2)
        pp = tr1.apply_pt(x2, y2)
        tr2 = trafo.trafo(matrix=((1, (pp[0]-1)), (0, 1)))
        p = p.transformed(tr2*tr1)
    c = canvas.canvas()
    c.stroke(p, [deco.shownormpath()])
    #c.text(-0.2, -10*unit.x_pt, r"\noindent$\Delta x=%g $\par\noindent$\Delta y=%g$"%(Dx,Dy),
    #       [text.parbox(4), text.size.footnotesize])
    return c

dx = 3
dy = -3

Dy = -2.5
e = 0.1
# test the left branch:
cc = acurve(pass00(Dy), Dy) # passes through startpoint
#cc = acurve(Dx_m(Dy)-e, Dy)
c  = acurve(Dx_m(Dy)  , Dy); cc.insert(c, [trafo.translate(1*dx, 0)])
c  = acurve(Dx_m(Dy)+e, Dy); cc.insert(c, [trafo.translate(2*dx, 0)])

# test the right branch:
#cc = acurve(Dx_p(Dy)-e, Dy, True)
#c  = acurve(Dx_p(Dy)  , Dy, True); cc.insert(c, [trafo.translate(1*dx, 0)])
##c  = acurve(Dx_p(Dy)+e, Dy, True); cc.insert(c, [trafo.translate(2*dx, 0)])
#c  = acurve(pass10(Dy), Dy, True); cc.insert(c, [trafo.translate(2*dx, 0)]) # passes through endpoint

cc.writePDFfile()




import sys; sys.path[:0] = ["../.."]
from pyx import *

c = canvas.canvas()

g = c.insert(graph.graphxy(ypos=6, width=7, y=graph.axis.log(parter=graph.axis.parter.log(tickpreexps=[graph.axis.parter.log.pre1exp, graph.axis.parter.log.pre1to9exp]))))
g.plot(graph.data.file("test_logaxis.dat", x=1, y=2))
g = c.insert(graph.graphxy(ypos=6, xpos=10, width=7, y=graph.axis.log(parter=graph.axis.parter.log(tickpreexps=[graph.axis.parter.log.pre1exp, graph.axis.parter.log.pre1to9exp]))))
g.plot(graph.data.file("test_logaxis.dat", x=1, y="-$2"))

g = c.insert(graph.graphxy(width=7, y=graph.axis.log()))
g.plot(graph.data.file("test_logaxis.dat", x=1, y=2))
g = c.insert(graph.graphxy(xpos=10, width=7, y=graph.axis.log()))
g.plot(graph.data.file("test_logaxis.dat", x=1, y="-$2"))

c.writeEPSfile()
c.writePDFfile()
c.writeSVGfile()



import sys
if sys.path[0] != "../..":
    sys.path.insert(0, "../..")

import io, os, re, unittest

from pyx.dvi import dvifile


class DvifileTestCase(unittest.TestCase):

    def dvitypetester(self, advifile):
        with os.popen("dvitype %s" % advifile) as dvitypefile:
            dvitypelines = dvitypefile.readlines()
        dvitypelineno = dvitypelines.index(" \n") + 1

        pyxdvifile = io.StringIO()
        df = dvifile.DVIfile(advifile, debug=1, debugfile=pyxdvifile)
        while df.readpage():
            pass
        pyxdvifilelines = list(pyxdvifile.getvalue().split("\n"))
        pyxdvifilelineno = 0

        while dvitypelineno < len(dvitypelines) and pyxdvifilelineno < len(pyxdvifilelines):
            dvitypeline = dvitypelines[dvitypelineno].rstrip()
            if dvitypeline.startswith("[") and dvitypeline.endswith("]"):
                dvitypelineno += 1
                continue

            pyxdvifileline = pyxdvifilelines[pyxdvifilelineno].rstrip()
            if pyxdvifileline.startswith("[") and pyxdvifileline.endswith("]"):
                pyxdvifilelineno += 1
                continue

            pyxdvifilelinere = (pyxdvifileline.replace("+", "\\+")
                                              .replace("(", "\\(")
                                              .replace(")", "\\)")
                                              .replace("???", "-?\\d+") +
                                              "( warning: \\|h\\|>\\d+!)?" +
                                              "$")
            if re.match(pyxdvifilelinere, dvitypeline):
                dvitypelineno += 1
                pyxdvifilelineno += 1
            else:
                raise ValueError("difference:\n%s\n%s" % (dvitypeline, pyxdvifileline))
                # print "difference:"
                # print "\t", dvitypeline
                # print "\t", pyxdvifileline
                # dvitypelineno += 1
                # pyxdvifilelineno += 1

        # don't be strict about empty tailing lines
        while dvitypelineno < len(dvitypelines) and not dvitypelines[dvitypelineno].strip():
            dvitypelineno += 1
        while pyxdvifilelineno < len(pyxdvifilelines) and not pyxdvifilelines[pyxdvifilelineno].strip():
            pyxdvifilelineno += 1

        assert dvitypelineno == len(dvitypelines)
        assert pyxdvifilelineno == len(pyxdvifilelines)

    def testDvitypeSample2e(self):
        os.system("latex sample2e.tex > /dev/null 2> /dev/null")
        self.dvitypetester("sample2e.dvi")
        os.system("rm sample2e.*")

    def testDvitypeBigScale(self):
        with open("bigscale.tex", "w") as texfile:
            texfile.write("\\nopagenumbers\n"
                          "\\font\\myfont=cmr10 at 145.678pt\\myfont\n"
                          "i\\par\n"
                          "\\font\\myfont=cmr10 at 457.12346pt\\myfont\n"
                          "m\\par\n"
                          "\\bye\n")
        os.system("tex bigscale.tex > /dev/null 2> /dev/null")
        self.dvitypetester("bigscale.dvi")
        os.system("rm bigscale.*")


if __name__ == "__main__":
    unittest.main()



from math import pi, sin, cos
from pyx import *

def vector(x1, y1, x2, y2, t, pos=0.5, distance=0.1):
    c = canvas.canvas()
    c.stroke(path.line(x1, y1, x2, y2), [deco.earrow.normal])
    textbox = text.text((1-pos)*x1 + pos*x2, (1-pos)*y1 + pos*y2, t,
                        [text.halign.center, text.vshift.mathaxis])
    if distance < 0:
        textbox.linealign(-distance, y1 - y2, x2 - x1)
    else:
        textbox.linealign(distance, y2 - y1, x1 - x2)
    c.insert(textbox)
    return c

r = 1.5
a = 150

c = canvas.canvas()
dx, dy = cos(a * pi / 180), sin(a * pi / 180)
x, y = r * dx, r * dy
c.stroke(path.circle(0, 0, r))
c.insert(vector(0, 0, x, y, r"$\vec r$"))
c.insert(vector(x, y, x - dy, y + dx, r"$\vec t$", pos=0.7))
c.insert(vector(x, y, x + dx, y + dy, r"$\vec n$", pos=0.7))
c.writeEPSfile()
c.writePDFfile()
c.writeSVGfile()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
#
# Copyright (C) 2005-2011 André Wobst <wobsta@pyx-project.org>
#
# dvitype is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# epstopng is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with epstopng; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from optparse import OptionParser
from pyx import version
from pyx.dvi import dvifile

parser = OptionParser(usage="usage: %prog dvi-file",
                      version="%prog " + version.version)
(options, args) = parser.parse_args()
if len(args) != 1:
    parser.error("can process a single dvi-file only")

df = dvifile.DVIfile(args[0], debug=True)
while df.readpage():
    pass



import sys
if sys.path[0] != "../..":
    sys.path.insert(0, "../..")

import unittest, warnings, os, logging

from testfixtures import log_capture

from pyx import text, unit

# text.set(texdebug="bla.tex", usefiles=["bla.log"])

class MessageParserTestCase(unittest.TestCase):

    @log_capture(level=logging.WARNING)
    def testBadTeX(self, l):
        text.text(0, 0, r"\some \badly \broken \TeX", texmessages=[text.texmessage.warn])
        l.check(("pyx", "WARNING", r"""ignoring TeX warnings:
  *
  *! Undefined control sequence.
  <argument> \some 
                   \badly \broken \TeX 
  <*> }{1}
          %
  ! Undefined control sequence.
  <argument> \some \badly 
                          \broken \TeX 
  <*> }{1}
          %
  ! Undefined control sequence.
  <argument> \some \badly \broken 
                                  \TeX 
  <*> }{1}
          %
  
  
  *"""))

    @log_capture(level=logging.WARNING)
    def testFontWarning(self, l):
        text.text(0, 0, r"\fontseries{invalid}\selectfont{}hello, world", texmessages=[text.texmessage.font_warning])
        text.defaulttextengine.instance.do_finish()
        l.check(("pyx", "WARNING", r"""ignoring font substitutions of NFSS:
LaTeX Font Warning: Font shape `OT1/cmr/invalid/n' undefined
(Font)              using `OT1/cmr/m/n' instead on input line 0."""),
                ("pyx", "WARNING", r"""ignoring font substitutions of NFSS:
LaTeX Font Warning: Some font shapes were not available, defaults substituted."""))

    @log_capture(level=logging.WARNING)
    def testOverfullHboxWarning(self, l):
        text.text(0, 0, r"hello, world", textattrs=[text.parbox(30*unit.u_pt)])
        l.check(("pyx", "WARNING", r"""ignoring overfull/underfull box:
Overfull \hbox (8.22089pt too wide) detected at line 0
[]\OT1/cmr/m/n/10 hello,"""))

    @log_capture(level=logging.WARNING)
    def testUnderfullHboxWarning(self, l):
        text.text(0, 0, r"\hbadness=0hello, world, hello", textattrs=[text.parbox(2.5)])
        l.check(("pyx", "WARNING", r"""ignoring overfull/underfull box:
Underfull \hbox (badness 171) detected at line 0
[]\OT1/cmr/m/n/10 hello, world,"""))

    @log_capture(level=logging.WARNING)
    def testOverfullVboxWarning(self, l):
        text.text(0, 0, r"\parindent=0pt\vbox to 1cm {hello, world, hello, world, hello, world}", textattrs=[text.parbox(1.9)])
        l.check(("pyx", "WARNING", r"""ignoring overfull/underfull box:
Overfull \vbox (2.4917pt too high) detected at line 0"""))

    @log_capture(level=logging.WARNING)
    def testUnderfullVboxWarning(self, l):
        text.text(0, 0, r"\parindent=0pt\vbox to 1cm {hello, world, hello, world}", textattrs=[text.parbox(1.9)])
        l.check(("pyx", "WARNING", r"""ignoring overfull/underfull box:
Underfull \vbox (badness 10000) detected at line 0"""))

    def testLoadLongFileNames(self):
        testfilename = "x"*100
        with open(testfilename + ".tex", "w") as f:
            f.write("\message{ignore this}")
        text.text(0, 0, "\\input %s\n" % testfilename, texmessages=[text.texmessage.load])
        os.remove(testfilename + ".tex")
        with open(testfilename + ".eps", "w") as f:
            f.write("%%BoundingBox: 0 0 10 10")
        text.text(0, 0, r"\includegraphics{%s}" % testfilename)
        os.remove(testfilename + ".eps")

    def setUp(self):
        text.set(engine=text.LatexEngine)
        text.preamble(r"\usepackage{graphicx}")

if __name__ == "__main__":
    unittest.main()



import sys; sys.path.insert(0, "../..")
from pyx import *

c = canvas.canvas()

l1b = c.layer("1.b")
c.stroke(path.line(0, 0, 1, 0))
l2 = c.layer("2")
l1a = c.layer("1.a", below="1.b")

l1b.text(0.22, 0, "1b", [color.rgb.red, text.halign.center, text.vshift.mathaxis])
l2.text(0.5, 0, "2", [color.rgb.green, text.halign.center, text.vshift.mathaxis])
l1a.text(0.75, 0, "1a", [color.rgb.blue, text.halign.center, text.vshift.mathaxis])
l1a.stroke(path.line(0.25, -0.25, 0.25, 0.25), [color.rgb.blue])

c.writeEPSfile("test_layer", page_paperformat=document.paperformat.A4, page_fittosize=1)
c.writePDFfile("test_layer", page_paperformat=document.paperformat.A4, page_fittosize=1)
c.writeSVGfile("test_layer", page_paperformat=document.paperformat.A4, page_fittosize=1)




from math import pi
from pyx import *

g = graph.graphxy(width=10, key=graph.key.key(),
                  x_divisor=pi, x_texter=graph.axis.texter.rational(suffix=r'\pi'),
                  y=graph.axis.split(reverse=1, subaxes={-1: graph.axis.log(min=-100),
                                                         1: graph.axis.log(max=100)}))
g.plot(graph.data.function("y(x)=(sgn(sin(x)), abs(sin(x)) > 0.001 and 1/sin(x) or None)",
                           min=0, max=6*pi, points=10001, title="y(x)=1/sin(x)"))
g.writeEPSfile()
g.writePDFfile()
g.writeSVGfile()



import sys
if sys.path[0] != "../..":
    sys.path.insert(0, "../..")

import unittest

import pyx
from pyx.attr import *

class A1(attr): pass
class B1(attr): pass
class C1(attr): pass

class A2(exclusiveattr): pass
class B2(exclusiveattr): pass
class C2(exclusiveattr): pass

class A3(sortbeforeattr): pass
class B3(sortbeforeattr): pass
class C3(attr): pass

class A4(sortbeforeexclusiveattr): pass
class B4(sortbeforeexclusiveattr): pass
class C4(attr): pass


class AttrTestCase(unittest.TestCase):

    def testCheck(self):
        checkattrs([A1(), B1(), A1()], [A1, B1])
        self.assertRaises(TypeError, checkattrs, [A1(), B1(), A1()], [A1, C1])

    def testMerge(self):
        a1 = A1()
        a2 = A1()
        b1 = B1()
        b2 = B1()
        c1 = C1()
        c2 = C1()
        self.assertEqual(mergeattrs([a1, b2, b1, c2, a2, c1]), [a1, b2, b1, c2, a2, c1])

    def testExclusive(self):
        a1 = A2(A2)
        a2 = A2(A2)
        b1 = B2(B2)
        b2 = B2(B2)
        c1 = C2(C2)
        c2 = C2(C2)
        self.assertEqual(mergeattrs([a1, b2, b1, c2, a2, c1]), [b1, a2, c1])

    def testSort(self):
        a1 = A3((B3, C3))
        a2 = A3((B3, C3))
        b1 = B3((C3,))
        b2 = B3((C3,))
        c1 = C3()
        c2 = C3()
        self.assertEqual(mergeattrs([a1, b2, b1, c2, a2, c1]), [a1, a2, b2, b1, c2, c1])

    def testExclusiveSort(self):
        a1 = A4(A4, (B4, C4))
        a2 = A4(A4, (B4, C4))
        b1 = B4(B4, (C4,))
        b2 = B4(B4, (C4,))
        c1 = C4()
        c2 = C4()
        self.assertEqual(mergeattrs([b2, a1, b1, c2, a2, c1]), [a2, b1, c2, c1])


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
import sys;sys.path.insert(0, "..")
from math import *
from pyx import *
# File for invalid parametrisations of Bezier curves
# visualise the constraints

def fx(x, sign1):
    if x < 1/3.0: # a_x > 0
        if sign1 == "upper":
            return -x + abs(x)
        else:
            return -x - abs(x)
    else:
        if sign1 == "upper":
            return x - abs(x)
        else:
            return x + abs(x)

def boundx(x):
    if x < 1/3.0:
        return 1-3*x
    else:
        return 3*x-1

def fy(y, sign2):
    if y < 0: # a_y > 0
        if sign2 == "upper":
            return 1-y + sqrt(1+y+y*y)
        else:
            return 1-y - sqrt(1+y+y*y)
    else:
        if sign2 == "upper":
            return y-1 - sqrt(1+y+y*y)
        else:
            return y-1 + sqrt(1+y+y*y)

def boundy(y):
    if y < 0:
        return -3*y
    else:
        return 3*y

def doplot(xtitle, con):
    g = graph.graphxy(width=10,
        x=graph.axis.lin(title=xtitle, min=-5, max=5),
        y=graph.axis.lin(),
        key=graph.key.key(pos="tc"),
    )
    g.plot(graph.data.function("y(x)=0", points=2, title=None), [graph.style.line()])
    g.plot(graph.data.function("y(x)=bound(x)", title=None, context=con, points=101), [graph.style.line()])
    g.plot(graph.data.function("y(x)=f(x, 'upper')", title="upper sign", context=con, points=101), [graph.style.line([color.rgb.red])])
    g.plot(graph.data.function("y(x)=f(x, 'lower')", title="lower sign", context=con, points=101), [graph.style.line([color.rgb.green])])
    return g


c1 = doplot(r"$\Delta x$", {"f":fx, "bound":boundx})
c2 = doplot(r"$\Delta y$", {"f":fy, "bound":boundy})
c1.insert(c2, [trafo.translate(0, c1.bbox().bottom() - c2.bbox().top() - 0.5)])
c1.writePDFfile()



import sys; sys.path.insert(0, "../..")

import math
from pyx import *
from pyx import normpath

# make math.sqrt raising an exception for negativ values
try:
    math.sqrt(-1)
except Exception:
    pass
else:
    math._sqrt = math.sqrt
    def mysqrt(x):
        if x < 0:
            raise ValueError("sqrt of negativ number")
        return math._sqrt(x)
    math.sqrt = mysqrt


c = canvas.canvas()

def connect(normpathitem1, normpathitem2, round):
    "returns a list of normsubpathitems connecting normpathitem1 and normpathitem2 either rounded or not"
    # this is for corners, i.e. when there are jumps in the tangent vector
    t1 = normpathitem1.trafo([1])[0]
    t2 = normpathitem2.trafo([0])[0]
    xs, ys = t1.apply_pt(0, 0)
    xe, ye = t2.apply_pt(0, 0)
    if (xs-xe)*(xs-xe) + (xs-xe)*(xs-xe) < 1e-5**2:
        return []
    if round:
        # calculate crossing point of normal
        try:
            param = ((t2.matrix[0][1]*(t1.vector[1]-t2.vector[1]) -
                      t2.matrix[1][1]*(t1.vector[0]-t2.vector[0])) /
                     (t2.matrix[1][1]*t1.matrix[0][1] -
                      t2.matrix[0][1]*t1.matrix[1][1]))
        except ArithmeticError:
            return [path.normline_pt(xs, ys, xe, ye)]
        else:
            # since t1 is a translation + rotation, param is the radius
            # hence, param should be equal to enlargeby_pt
            # print param
            x, y = t1.apply_pt(0, param)
            # c.fill(path.circle_pt(x, y, 1))
            angle1 = math.atan2(ys-y, xs-x)
            angle2 = math.atan2(ye-y, xe-x)
            while angle2 < angle1:
                angle1 -= 2*math.pi
            return [path._arctobcurve(x, y, param, angle1, angle2)]
    else:
        # calculate crossing point of tangents
        try:
            param = ((t2.matrix[0][0]*(t1.vector[1]-t2.vector[1]) -
                      t2.matrix[1][0]*(t1.vector[0]-t2.vector[0])) / 
                     (t2.matrix[1][0]*t1.matrix[0][0] -
                      t2.matrix[0][0]*t1.matrix[1][0]))
        except ArithmeticError:
            return [path.normline_pt(xs, ys, xe, ye)]
        else:
            x, y = t1.apply_pt(param, 0)
            return [path.normline_pt(xs, ys, x, y), path.normline_pt(x, y, xe, ye)]


def enlarged(anormpath, enlargeby_pt, round):
    newnormsubpaths = []
    for normsubpath in anormpath.normsubpaths:
        splitnormsubpathitems = normsubpath.normsubpathitems[:] # do splitting on a copy
        i = 0
        while i < len(splitnormsubpathitems):
            if isinstance(splitnormsubpathitems[i], normpath.normcurve_pt) and splitnormsubpathitems[i].arclen_pt(normsubpath.epsilon) > 100:
                splitnormsubpathitems[i:i+1] = splitnormsubpathitems[i]._midpointsplit(normsubpath.epsilon)
            else:
                i += 1
        newnormsubpathitems = []
        for normsubpathitem in splitnormsubpathitems:
            # get old and new start and end points
            ts, te = normsubpathitem.trafo([0, 1])
            xs, ys = ts.apply_pt(0, 0)
            nxs, nys = ts.apply_pt(0, -enlargeby_pt)
            xe, ye = te.apply_pt(0, 0)
            nxe, nye = te.apply_pt(0, -enlargeby_pt)

            if isinstance(normsubpathitem, normpath.normcurve_pt):
                # We should do not alter the sign. Could we do any better here?
                try:
                    cs = 1/(normsubpathitem.curveradius_pt([0])[0] + enlargeby_pt)
                except ArithmeticError:
                    cs = 0
                try:
                    ce = 1/(normsubpathitem.curveradius_pt([1])[0] + enlargeby_pt)
                except ArithmeticError:
                    ce = 0

                # this should be a function (in path?), not a method in a class
                # the parameter convention is quite different from other places ...
                bezierparams = deformer.normcurve_from_endgeometry_pt((nxs, nys), (nxe, nye),
                                                                      (ts.matrix[0][0], ts.matrix[1][0]),
                                                                      (te.matrix[0][0], te.matrix[1][0]),
                                                                      cs, ce)
                c.fill(path.circle_pt(bezierparams.x0_pt, bezierparams.y0_pt, 1), [color.rgb.blue])
                c.fill(path.circle_pt(bezierparams.x3_pt, bezierparams.y3_pt, 1), [color.rgb.blue])
                newnormpathitems = normpath.normcurve_pt(bezierparams.x0_pt, bezierparams.y0_pt,
                                               bezierparams.x1_pt, bezierparams.y1_pt,
                                               bezierparams.x2_pt, bezierparams.y2_pt,
                                               bezierparams.x3_pt, bezierparams.y3_pt)
                showtangent(newnormpathitems) # line alignment of bezier curves
                showcircle(newnormpathitems)  # circle alignment of bezier curves
            else:
                # line
                newnormpathitems = path.normline_pt(nxs, nys, nxe, nye)

            if len(newnormsubpathitems):
                newnormsubpathitems.extend(connect(newnormsubpathitems[-1], newnormpathitems, round=round))
            newnormsubpathitems.append(newnormpathitems)
        if normsubpath.closed:
            newnormsubpathitems.extend(connect(newnormsubpathitems[-1], newnormsubpathitems[0], round=round))
        newnormsubpaths.append(path.normsubpath(newnormsubpathitems, normsubpath.closed))
    return normpath.normpath(newnormsubpaths)


def showtangent(normcurve_pt):
    dx = 2
    dy = 1

    cx =                                       3*normcurve_pt.x1_pt - 3*normcurve_pt.x0_pt
    bx =                   3*normcurve_pt.x2_pt - 6*normcurve_pt.x1_pt + 3*normcurve_pt.x0_pt
    ax = normcurve_pt.x3_pt - 3*normcurve_pt.x2_pt + 3*normcurve_pt.x1_pt -   normcurve_pt.x0_pt

    cy =                                       3*normcurve_pt.y1_pt - 3*normcurve_pt.y0_pt
    by =                   3*normcurve_pt.y2_pt - 6*normcurve_pt.y1_pt + 3*normcurve_pt.y0_pt
    ay = normcurve_pt.y3_pt - 3*normcurve_pt.y2_pt + 3*normcurve_pt.y1_pt -   normcurve_pt.y0_pt

    try:
        x = - (dy*bx-dx*by)/(3.0*dy*ax-3.0*dx*ay)
        s = math.sqrt(x*x-(dy*cx-dx*cy)/(3.0*dy*ax-3.0*dx*ay))
    except (ArithmeticError, ValueError):
        return
    t1 = x-s
    t2 = x+s
    ts = []
    if 0 < t1 < 1:
        ts.append(t1)
    if 0 < t2 < 1:
        ts.append(t2)
    for t in ts:
        trafo = normcurve_pt.trafo([t])[0]
        x, y = trafo.apply_pt(0, 0)
        c.fill(path.circle_pt(x, y, 1), [color.rgb.red])
        c.stroke(path.line_pt(*list(trafo.apply_pt(-100, 0))+list(trafo.apply_pt(100, 0))), [color.rgb.red])

def showcircle(normcurve_pt):
    gx = 350
    gy = 200
    hx = -1
    hy = 2
    cos = hx/math.sqrt(hx*hx+hy*hy)
    sin = hy/math.sqrt(hx*hx+hy*hy)

    r = 16

    dx =                                                             normcurve_pt.x0_pt
    cx =                                       3*normcurve_pt.x1_pt - 3*normcurve_pt.x0_pt
    bx =                   3*normcurve_pt.x2_pt - 6*normcurve_pt.x1_pt + 3*normcurve_pt.x0_pt
    ax = normcurve_pt.x3_pt - 3*normcurve_pt.x2_pt + 3*normcurve_pt.x1_pt -   normcurve_pt.x0_pt

    dy =                                                             normcurve_pt.y0_pt
    cy =                                       3*normcurve_pt.y1_pt - 3*normcurve_pt.y0_pt
    by =                   3*normcurve_pt.y2_pt - 6*normcurve_pt.y1_pt + 3*normcurve_pt.y0_pt
    ay = normcurve_pt.y3_pt - 3*normcurve_pt.y2_pt + 3*normcurve_pt.y1_pt -   normcurve_pt.y0_pt

    def x(t, gx=gx):
        return ax*t*t*t+bx*t*t+cx*t+normcurve_pt.x0_pt
    def y(t, gy=gy):
        return ay*t*t*t+by*t*t+cy*t+normcurve_pt.y0_pt
    def xdot(t):
        return 3*ax*t*t+2*bx*t+cx
    def ydot(t):
        return 3*ay*t*t+2*by*t+cy

    # we need to find roots of the following polynom:
    # (xdot(t)**2+ydot(t)**2)*((x(t)-gx)*hy-(y(t)-gy)*hx)**2-r*r*(xdot(t)*hx+ydot(t)*hy)**2 == 0

    # the polynom is of order 10. its coefficients are:

    coeffs = [9*(ax**2+ay**2)*(hy*ax-ay*hx)**2, # * t**10,

              -6*(hy*ax-ay*hx)*(-5*ax**2*bx*hy+3*ax**2*hx*by-2*ax*by*ay*hy+
              2*ax*hx*bx*ay-3*ay**2*bx*hy+5*hx*ay**2*by), # * t**9, etc.

              -30*hy*hx*ay*ax**2*cx-32*hy*hx*ay*bx**2*ax-42*hy*hx*bx*ax**2*by-18*hy*hx*ax**3*cy+
              24*hy**2*ax**3*cx+37*hy**2*bx**2*ax**2+24*hx**2*ay**3*cy+37*hx**2*by**2*ay**2-
              30*hy*hx*ay**2*ax*cy-32*hy*hx*ay*ax*by**2-42*hy*hx*by*ay**2*bx-18*hy*hx*ay**3*cx+
              6*hy**2*ax**2*cy*ay+4*hy**2*ax**2*by**2+24*hy**2*bx*ax*by*ay+18*hy**2*ay**2*cx*ax+
              9*hy**2*ay**2*bx**2+6*hx**2*ay**2*cx*ax+4*hx**2*ay**2*bx**2+24*hx**2*bx*ax*by*ay+
              18*hx**2*ax**2*cy*ay+9*hx**2*ax**2*by**2,

              20*hx**2*by**3*ay+8*hx**2*by*ay*bx**2+4*hx**2*cx*bx*ay**2-18*hx**2*ax**2*ay*gy+
              18*hx**2*ax**2*ay*dy+20*hy**2*bx**3*ax+18*hy**2*ax**3*dx+12*hx**2*by*ay*cx*ax+18*
              hx**2*cy*by*ax**2+12*hx**2*bx*ax*by**2+8*hy**2*bx*ax*by**2+4*hy**2*cy*by*ax**2-
              18*hy**2*ay**2*ax*gx+18*hy**2*ay**2*ax*dx+18*hy**2*cx*bx*ay**2+12*hy**2*by*ay*bx**2
              +58*hy**2*cx*bx*ax**2-8*hy*hx*ay*bx**3+18*hy*hx*ax**3*gy-18*hy*hx*ax**3*dy-
              18*hy*hx*ay**3*dx-8*hy*hx*ax*by**3+58*hx**2*cy*by*ay**2+24*hx**2*bx*ax*cy*ay+
              12*hy**2*bx*ax*cy*ay+24*hy**2*by*ay*cx*ax-18*hy**2*ax**3*gx-44*hy*hx*ay*cx*bx*ax+
              18*hy*hx*ay*ax**2*gx-18*hy*hx*ay*ax**2*dx-30*hy*hx*by*ax**2*cx-32*hy*hx*by*bx**2*ax-
              42*hy*hx*bx*ax**2*cy+18*hy*hx*ay**3*gx-44*hy*hx*ay*cy*by*ax-30*hy*hx*bx*cy*ay**2-
              32*hy*hx*ay*bx*by**2-42*hy*hx*by*ay**2*cx+18*hy*hx*ay**2*ax*gy-
              18*hy*hx*ay**2*ax*dy-18*hx**2*ay**3*gy+18*hx**2*ay**3*dy,

              hx**2*cx**2*ay**2+46*hy**2*cx*bx**2*ax-42*hy**2*bx*ax**2*gx+42*hy**2*bx*ax**2*dx+
              46*hx**2*cy*by**2*ay-42*hx**2*by*ay**2*gy+42*hx**2*by*ay**2*dy-8*hy*hx*bx*by**3+
              6*hy**2*cy*ay*bx**2+8*hy**2*by**2*cx*ax-18*hy**2*ay**2*bx*gx+18*hy**2*ay**2*bx*dx+
              6*hx**2*by**2*cx*ax+8*hx**2*cy*ay*bx**2-18*hx**2*ax**2*by*gy+18*hx**2*ax**2*by*dy-
              8*hy*hx*by*bx**3+4*hy**2*bx**4+22*hy**2*cx**2*ax**2+22*hx**2*cy**2*ay**2+hy**2*cy**2*ax**2+
              4*hy**2*by**2*bx**2+9*hy**2*cx**2*ay**2+4*hx**2*by**2*bx**2+9*hx**2*cy**2*ax**2+
              4*hx**2*by**4-14*hy*hx*ay*cy**2*ax-44*hy*hx*ay*cy*by*bx-30*hy*hx*cx*cy*ay**2-
              32*hy*hx*ay*cx*by**2+42*hy*hx*by*ay**2*gx-42*hy*hx*by*ay**2*dx-16*hy*hx*cy*by**2*ax+
              24*hy*hx*by*ay*ax*gy-24*hy*hx*by*ay*ax*dy+18*hy*hx*ay**2*bx*gy-18*hy*hx*ay**2*bx*dy+
              8*hy**2*cy*by*bx*ax+12*hy**2*cy*ay*cx*ax-24*hy**2*by*ay*ax*gx+24*hy**2*by*ay*ax*dx+
              24*hy**2*cx*bx*by*ay+8*hx**2*cx*bx*by*ay+12*hx**2*cy*ay*cx*ax-24*hx**2*bx*ax*ay*gy+
              24*hx**2*bx*ax*ay*dy+24*hx**2*cy*by*bx*ax-14*hy*hx*ay*cx**2*ax-16*hy*hx*ay*cx*bx**2+
              24*hy*hx*ay*bx*ax*gx-24*hy*hx*ay*bx*ax*dx-44*hy*hx*by*cx*bx*ax+18*hy*hx*by*ax**2*gx-
              18*hy*hx*by*ax**2*dx-30*hy*hx*cy*ax**2*cx-32*hy*hx*cy*bx**2*ax+42*hy*hx*bx*ax**2*gy-
              42*hy*hx*bx*ax**2*dy,

              12*hx**2*cy*by**3+12*hy**2*cx*bx**3+34*hx**2*cy**2*by*ay-30*hx**2*cy*ay**2*gy+
              30*hx**2*cy*ay**2*dy-32*hx**2*by**2*ay*gy+32*hx**2*by**2*ay*dy+2*hx**2*cx**2*by*ay+
              4*hx**2*cx*bx*by**2-8*hx**2*bx**2*ay*gy+8*hx**2*bx**2*ay*dy+8*hx**2*bx**2*cy*by+
              12*hx**2*cy**2*bx*ax-18*hx**2*ax**2*cy*gy+8*hx**2*cx*bx*cy*ay-12*hx**2*cx*ax*ay*gy+
              12*hx**2*cx*ax*ay*dy+12*hx**2*cx*ax*cy*by-24*hx**2*bx*ax*by*gy+24*hx**2*bx*ax*by*dy+
              18*hx**2*ax**2*cy*dy-8*hy*hx*cy*bx**3-8*hy*hx*cx*by**3+34*hy**2*cx**2*bx*ax-
              30*hy**2*cx*ax**2*gx+30*hy**2*cx*ax**2*dx-32*hy**2*bx**2*ax*gx+32*hy**2*bx**2*ax*dx+
              2*hy**2*cy**2*bx*ax+4*hy**2*bx**2*cy*by-8*hy**2*by**2*ax*gx+8*hy**2*by**2*ax*dx+
              8*hy**2*cx*bx*by**2+12*hy**2*cx**2*by*ay-18*hy**2*ay**2*cx*gx+18*hy**2*ay**2*cx*dx-
              10*hy*hx*ay*cx**2*bx+12*hy*hx*ay*cx*ax*gx-12*hy*hx*ay*cx*ax*dx+
              8*hy*hx*ay*bx**2*gx-8*hy*hx*ay*bx**2*dx-14*hy*hx*by*cx**2*ax-16*hy*hx*by*cx*bx**2+
              24*hy*hx*by*bx*ax*gx-24*hy*hx*by*bx*ax*dx-44*hy*hx*cy*cx*bx*ax+18*hy*hx*cy*ax**2*gx-
              18*hy*hx*cy*ax**2*dx+30*hy*hx*ax**2*cx*gy-30*hy*hx*ax**2*cx*dy+32*hy*hx*bx**2*ax*gy-
              32*hy*hx*bx**2*ax*dy-32*hy*hx*ay*by**2*dx-16*hy*hx*cy*by**2*bx+8*hy*hx*ax*by**2*gy+
              32*hy*hx*ay*by**2*gx-44*hy*hx*ay*cy*by*cx+12*hy*hx*ax*cy*ay*gy-12*hy*hx*ax*cy*ay*dy+
              24*hy*hx*by*ay*bx*gy-24*hy*hx*by*ay*bx*dy+30*hy*hx*cy*ay**2*gx-30*hy*hx*cy*ay**2*dx-
              14*hy*hx*ay*cy**2*bx-10*hy*hx*by*cy**2*ax-8*hy*hx*ax*by**2*dy+18*hy*hx*ay**2*cx*gy-
              18*hy*hx*ay**2*cx*dy+8*hy**2*cx*ax*cy*by-12*hy**2*cy*ay*ax*gx+12*hy**2*cy*ay*ax*dx+
              12*hy**2*cx*bx*cy*ay-24*hy**2*by*ay*bx*gx+24*hy**2*by*ay*bx*dx,

              hy**2*bx**2*cy**2+13*hy**2*cx**2*bx**2+9*hx**2*ax**2*gy**2-9*r**2*ax**2*hx**2-
              9*r**2*ay**2*hy**2+9*hx**2*ax**2*dy**2+hx**2*cx**2*by**2+4*hx**2*bx**2*cy**2+8*hy**2*cx**3*ax+
              8*hx**2*cy**3*ay+13*hx**2*cy**2*by**2-8*hx**2*by**3*gy+8*hx**2*by**3*dy+9*hx**2*ay**2*gy**2+
              9*hx**2*ay**2*dy**2+4*hy**2*cx**2*by**2+9*hy**2*ay**2*gx**2+9*hy**2*ay**2*dx**2-8*hy**2*bx**3*gx+
              8*hy**2*bx**3*dx+9*hy**2*ax**2*gx**2+9*hy**2*ax**2*dx**2-18*hx**2*ay**2*gy*dy+2*hx**2*cx**2*cy*ay+
              6*hx**2*cx*ax*cy**2-8*hx**2*bx**2*by*gy+8*hx**2*bx**2*by*dy-18*hx**2*ax**2*gy*dy+8*hy*hx*by**3*gx-
              8*hy*hx*by**3*dx-2*hy*hx*cy**3*ax+2*hy**2*cx*ax*cy**2+6*hy**2*cx**2*cy*ay-8*hy**2*by**2*bx*gx+
              8*hy**2*by**2*bx*dx-18*hy**2*ay**2*gx*dx-18*hy**2*ax**2*gx*dx-18*r**2*ax*ay*hx*hy-
              44*hx**2*cy*by*ay*gy+44*hx**2*cy*by*ay*dy-8*hx**2*cx*bx*ay*gy+8*hx**2*cx*bx*ay*dy+
              8*hx**2*cx*bx*cy*by-12*hx**2*cx*ax*by*gy+12*hx**2*cx*ax*by*dy-24*hx**2*bx*ax*cy*gy+
              24*hx**2*bx*ax*cy*dy+8*hy*hx*bx*by**2*gy-8*hy*hx*bx*by**2*dy+18*hy*hx*ay**2*dx*gy-
              14*hy*hx*ay*cy**2*cx-18*hy*hx*ay**2*gx*gy-18*hy*hx*ay**2*dx*dy-16*hy*hx*cy*by**2*cx+
              18*hy*hx*ay**2*gx*dy-10*hy*hx*by*cy**2*bx+44*hy*hx*ay*cy*by*gx-44*hy*hx*ay*cy*by*dx+
              8*hy*hx*cy*by*ax*gy-8*hy*hx*cy*by*ax*dy+12*hy*hx*bx*cy*ay*gy-12*hy*hx*bx*cy*ay*dy+
              24*hy*hx*by*ay*cx*gy-24*hy*hx*by*ay*cx*dy-8*hy**2*cy*by*ax*gx+8*hy**2*cy*by*ax*dx+
              8*hy**2*cx*bx*cy*by-12*hy**2*cy*ay*bx*gx+12*hy**2*cy*ay*bx*dx-24*hy**2*by*ay*cx*gx+
              24*hy**2*by*ay*cx*dx-44*hy**2*cx*bx*ax*gx+44*hy**2*cx*bx*ax*dx-2*hy*hx*ay*cx**3+
              8*hy*hx*bx**3*gy-8*hy*hx*bx**3*dy+8*hy*hx*ay*cx*bx*gx-8*hy*hx*ay*cx*bx*dx-
              10*hy*hx*by*cx**2*bx+12*hy*hx*by*cx*ax*gx-12*hy*hx*by*cx*ax*dx+8*hy*hx*by*bx**2*gx-
              8*hy*hx*by*bx**2*dx-14*hy*hx*cy*cx**2*ax-16*hy*hx*cy*cx*bx**2+24*hy*hx*cy*bx*ax*gx-
              24*hy*hx*cy*bx*ax*dx+44*hy*hx*cx*bx*ax*gy-44*hy*hx*cx*bx*ax*dy-18*hy*hx*ax**2*gx*gy+
              18*hy*hx*ax**2*gx*dy+18*hy*hx*ax**2*dx*gy-18*hy*hx*ax**2*dx*dy,

              6*hx**2*cy**3*by-24*hx**2*by*ay*gy*dy-8*hy**2*cy*by*bx*gx-14*hy**2*cx**2*ax*gx-
              12*hy*hx*r**2*bx*ay+2*hy*hx*cx**2*ay*gx-12*r**2*bx*ax*hx**2-12*r**2*by*ay*hy**2+
              16*hy**2*cx*bx**2*dx+6*hy**2*cx**3*bx-16*hy**2*cx*bx**2*gx+14*hy**2*cx**2*ax*dx+
              12*hy**2*bx*ax*gx**2+12*hy**2*bx*ax*dx**2-2*hy*hx*by*cx**3+4*hx**2*cx*bx*cy**2-
              2*hx**2*cx**2*ay*gy+2*hx**2*cx**2*ay*dy+2*hx**2*cx**2*cy*by+12*hx**2*bx*ax*gy**2+
              12*hx**2*bx*ax*dy**2-8*hx**2*cy*bx**2*gy+8*hx**2*cy*bx**2*dy+16*hx**2*cy*by**2*dy-
              14*hx**2*cy**2*ay*gy+14*hx**2*cy**2*ay*dy+12*hx**2*by*ay*gy**2+12*hx**2*by*ay*dy**2+
              4*hy**2*cx**2*cy*by-2*hy**2*cy**2*ax*gx+2*hy**2*cy**2*ax*dx+2*hy**2*cx*bx*cy**2+
              12*hy**2*by*ay*gx**2+12*hy**2*by*ay*dx**2-8*hy**2*cx*by**2*gx+8*hy**2*cx*by**2*dx-
              24*hy**2*bx*ax*gx*dx-2*hy*hx*cx**2*ay*dx+8*hy*hx*by*cx*bx*gx-8*hy*hx*by*cx*bx*dx-
              10*hy*hx*cy*cx**2*bx+12*hy*hx*cy*cx*ax*gx-12*hy*hx*cy*cx*ax*dx+8*hy*hx*cy*bx**2*gx-
              8*hy*hx*cy*bx**2*dx+14*hy*hx*cx**2*ax*gy-14*hy*hx*cx**2*ax*dy+16*hy*hx*cx*bx**2*gy-
              16*hy*hx*cx*bx**2*dy-24*hy*hx*bx*ax*gx*gy+24*hy*hx*bx*ax*gx*dy+24*hy*hx*bx*ax*dx*gy-
              24*hy*hx*bx*ax*dx*dy-2*hy*hx*cy**3*bx+14*hy*hx*cy**2*ay*gx-14*hy*hx*cy**2*ay*dx-
              10*hy*hx*by*cy**2*cx+16*hy*hx*cy*by**2*gx-16*hy*hx*cy*by**2*dx+2*hy*hx*cy**2*ax*gy-
              2*hy*hx*cy**2*ax*dy+8*hy*hx*cy*by*bx*gy-8*hy*hx*cy*by*bx*dy+12*hy*hx*cx*cy*ay*gy-
              12*hy*hx*cx*cy*ay*dy+8*hy*hx*cx*by**2*gy-8*hy*hx*cx*by**2*dy-24*hy*hx*by*ay*gx*gy+
              24*hy*hx*by*ay*gx*dy+24*hy*hx*by*ay*dx*gy-24*hy*hx*by*ay*dx*dy-8*hx**2*cx*bx*by*gy+
              8*hx**2*cx*bx*by*dy-24*hx**2*bx*ax*gy*dy-12*hx**2*cy*cx*ax*gy+12*hx**2*cy*cx*ax*dy-
              16*hx**2*cy*by**2*gy+8*hy**2*cy*by*bx*dx-24*hy**2*by*ay*gx*dx-12*hy**2*cx*cy*ay*gx+
              12*hy**2*cx*cy*ay*dx-12*hy*hx*r**2*ax*by,

              hy**2*cy**2*cx**2-4*r**2*hy**2*by**2+4*hx**2*bx**2*dy**2+4*hx**2*by**2*gy**2-
              4*r**2*hx**2*bx**2+4*hy**2*by**2*dx**2+4*hy**2*by**2*gx**2+4*hx**2*by**2*dy**2+
              hx**2*cy**2*cx**2+4*hy**2*bx**2*dx**2+8*hy**2*cy*by*cx*dx+4*hx**2*bx**2*gy**2+
              2*hy*hx*cy**2*bx*gy+4*hy**2*bx**2*gx**2-2*hy*hx*cy**2*bx*dy-10*hy*hx*cy**2*by*dx-
              8*hy**2*cy*by*cx*gx+10*hy*hx*cy**2*by*gx-6*hy*hx*r**2*cx*ay-10*hx**2*cy**2*by*gy+
              10*hx**2*cy**2*by*dy+6*hx**2*cy*ay*gy**2+6*hx**2*cy*ay*dy**2-8*hx**2*by**2*gy*dy-
              10*hy**2*cx**2*bx*gx+10*hy**2*cx**2*bx*dx+6*hy**2*cx*ax*gx**2+6*hy**2*cx*ax*dx**2-
              8*hy**2*bx**2*gx*dx-6*r**2*hx**2*cx*ax-2*hy**2*cy**2*bx*gx+2*hy**2*cy**2*bx*dx+
              6*hy**2*cy*ay*gx**2+6*hy**2*cy*ay*dx**2-8*hy**2*by**2*gx*dx-2*hy*hx*cy**3*cx-
              6*r**2*hy**2*cy*ay-2*hy*hx*cy*cx**3-2*hx**2*cx**2*by*gy+2*hx**2*cx**2*by*dy+
              6*hx**2*cx*ax*gy**2+6*hx**2*cx*ax*dy**2-8*hx**2*bx**2*gy*dy+hx**2*cy**4+
              hy**2*cx**4-12*hx**2*cy*ay*gy*dy-12*hy**2*cx*ax*gx*dx-8*hy*hx*r**2*bx*by-
              6*hy*hx*r**2*ax*cy-12*hy**2*cy*ay*gx*dx+8*hy*hx*cy*by*cx*gy-8*hy*hx*cy*by*cx*dy-
              12*hy*hx*cy*ay*gx*gy+12*hy*hx*cy*ay*gx*dy+12*hy*hx*cy*ay*dx*gy-
              12*hy*hx*cy*ay*dx*dy-8*hy*hx*by**2*gx*gy+8*hy*hx*by**2*gx*dy+8*hy*hx*by**2*dx*gy-
              8*hy*hx*by**2*dx*dy+2*hy*hx*cx**2*by*gx-2*hy*hx*cx**2*by*dx+8*hy*hx*cy*cx*bx*gx-
              8*hy*hx*cy*cx*bx*dx+10*hy*hx*cx**2*bx*gy-10*hy*hx*cx**2*bx*dy-12*hy*hx*cx*ax*gx*gy+
              12*hy*hx*cx*ax*gx*dy+12*hy*hx*cx*ax*dx*gy-12*hy*hx*cx*ax*dx*dy-8*hy*hx*bx**2*gx*gy+
              8*hy*hx*bx**2*gx*dy+8*hy*hx*bx**2*dx*gy-8*hy*hx*bx**2*dx*dy-8*hx**2*cx*bx*cy*gy+
              8*hx**2*cx*bx*cy*dy-12*hx**2*cx*ax*gy*dy,

              -2*hx**2*cy**3*gy-2*hy**2*cx**3*gx+2*hy**2*cx**3*dx-4*hy*hx*r**2*cx*by+
              2*hy*hx*cy*cx**2*gx+2*hx**2*cy**3*dy-4*r**2*cx*bx*hx**2-4*r**2*cy*by*hy**2-
              2*hy**2*cy**2*cx*gx+2*hy**2*cy**2*cx*dx+4*hy**2*cy*by*gx**2+4*hy**2*cy*by*dx**2+
              4*hx**2*cy*by*gy**2+4*hx**2*cy*by*dy**2+2*hy*hx*cx**3*gy-2*hy*hx*cx**3*dy+
              4*hy**2*cx*bx*gx**2+4*hy**2*cx*bx*dx**2-2*hx**2*cy*cx**2*gy+2*hx**2*cy*cx**2*dy+
              4*hx**2*cx*bx*gy**2+4*hx**2*cx*bx*dy**2+2*hy*hx*cy**3*gx-2*hy*hx*cy**3*dx-
              8*hy**2*cy*by*gx*dx-4*hy*hx*r**2*bx*cy-8*hx**2*cy*by*gy*dy-2*hy*hx*cy*cx**2*dx-
              8*hy*hx*cx*bx*gx*gy+8*hy*hx*cx*bx*gx*dy+8*hy*hx*cx*bx*dx*gy-
              8*hy*hx*cx*bx*dx*dy-8*hy**2*cx*bx*gx*dx-8*hx**2*cx*bx*gy*dy+2*hy*hx*cy**2*cx*gy-
              2*hy*hx*cy**2*cx*dy-8*hy*hx*cy*by*gx*gy+8*hy*hx*cy*by*gx*dy+
              8*hy*hx*cy*by*dx*gy-8*hy*hx*cy*by*dx*dy,

              cx**2*hy**2*gx**2-2*cx**2*hy**2*gx*dx+cx**2*hy**2*dx**2-2*r**2*hx*hy*cx*cy+
              cy**2*hx**2*gy**2-2*cy**2*hx**2*gy*dy+cy**2*hx**2*dy**2+cy**2*hy**2*gx**2-
              2*cy**2*hy**2*gx*dx+cy**2*hy**2*dx**2-2*cy**2*hx*hy*gy*gx+2*cy**2*hx*hy*dy*gx+
              2*cy**2*hx*hy*gy*dx-2*cy**2*hx*hy*dy*dx+cx**2*hx**2*gy**2-2*cx**2*hx**2*gy*dy+
              cx**2*hx**2*dy**2-r**2*hy**2*cy**2-2*cx**2*hx*hy*gy*gx+2*cx**2*hx*hy*dy*gx+
              2*cx**2*hx*hy*gy*dx-2*cx**2*hx*hy*dy*dx-r**2*hx**2*cx**2]

    # def z(t):
    #     res = 0
    #     for x in coeffs:
    #         res = res*t + x
    #     return res

    # def zdot(t):
    #     res = 0
    #     i = len(coeffs)
    #     for x in coeffs[:-1]:
    #         res = res*t + (i-1)*x
    #         i -= 1
    #     return res

    # def newton(t):
    #     # newton iteration to find the zero of z with t being the starting value for t
    #     if t < 0: t = 0
    #     if t > 1: t = 1
    #     slow = 1
    #     try:
    #         isz=z(t)
    #     except (ValueError, ArithmeticError):
    #         return
    #     loop = 0
    #     while abs(isz) > 1e-6:
    #         try:
    #             nt = t - slow * isz/zdot(t)
    #             newisz = z(nt)
    #         except (ValueError, ArithmeticError):
    #             slow *= 0.5 # we might have slow down the newton iteration
    #         else:
    #             t = nt
    #             isz = newisz
    #             if slow <= 0.5:
    #                 slow *= 2
    #         if loop > 100:
    #             break
    #         loop += 1
    #     else:
    #         return t

    # def ts():
    #     epsilon = 1e-5
    #     res = []
    #     count = 100
    #     for x in range(count+1):
    #        t = newton(x/float(count))
    #        if t is not None and 0-0.5*epsilon < t < 1+0.5*epsilon:
    #            res.append(t)
    #     res.sort()
    #     i = 1
    #     while len(res) > i:
    #         if res[i] > res[i-1] + epsilon:
    #             i += 1
    #         else:
    #             del res[i]
    #     return res

    # use Numeric to find the roots (via an equivalent eigenvalue problem)
    import Numeric, LinearAlgebra
    mat = Numeric.zeros((10, 10), Numeric.Float)
    for i in range(9):
        mat[i+1][i] = 1
    for i in range(10):
        mat[0][i] = -coeffs[i+1]/coeffs[0]
    ists = [zero.real for zero in LinearAlgebra.eigenvalues(mat) if -1e-10 < zero.imag < 1e-10 and 0 <= zero.real <= 1]

    for t in ists:
        isl = (xdot(t)*(x(t)-gx)+ydot(t)*(y(t)-gy))/(xdot(t)*cos+ydot(t)*sin)
        c.fill(path.circle_pt(x(t), y(t), 1), [color.rgb.green])
        c.stroke(path.circle_pt(gx+isl*cos, gy+isl*sin, r), [color.rgb.green])


# parameters for the enlargement:
enlargeby_pt = 10
round = 0 # a boolean to turn round corners on and off

# some examples:
for anormpath in [path.rect(0, 0, 5, 5).normpath(),
                 path.circle(10, 3, 3).normpath(),
                 path.path(path.moveto(0, 8),
                           path.curveto(1, 8, 1, 9, 2, 11),
                           path.curveto(1, 12, 1, 12, 0, 11),
                           path.lineto(0, 8),
                           path.closepath()).normpath(),
                 path.path(path.moveto(5, 8),
                           path.curveto(20, 8, 6, 9, 7, 11),
                           path.curveto(6, 12, 6, 12, 5, 11),
                           path.lineto(5, 8),
                           path.closepath()).normpath(),
                 path.path(path.moveto(16, 0),
                           path.curveto(18, 0, 18, 4, 20, 4),
                           path.lineto(20, 12),
                           path.curveto(12, 12, 20, 8, 16, 8),
                           path.closepath()).normpath()]:
    c.stroke(anormpath)
    c.stroke(enlarged(anormpath, enlargeby_pt, round), [color.rgb.blue])

c.writeEPSfile("newbox")



# contributed by Stefan Schenk

from io import BytesIO
from math import sqrt
from pyx import *

xiterations = yiterations = 250
Min_Im = -1.5
Max_Im = 1.5
Min_Re = -1.5
Max_Re = 1.5
c = 0.41 + 0.3j
p = color.gradient.RedBlue

def rgbcolortostring(c):
    return bytes([int(255*c.r), int(255*c.g), int(255*c.b)])

data = BytesIO()

# compute fractal
for y in range(yiterations):
    for x in range(xiterations):
        z = complex(1.0*(Max_Re-Min_Re)*x/xiterations + Min_Re,
                    1.0*(Max_Im-Min_Im)*y/yiterations + Min_Im)

        for k in range(256):
            z = z*z + c
            if abs(z) > 2:
                # append color(RGB) of the current pixel to the end of data
                data.write(rgbcolortostring(p.getcolor(1.0/sqrt(k+1))))
                break
        else:
            data.write(b"\0\0\0")

# generate image from data
julia = bitmap.image(xiterations, yiterations, "RGB", data.getvalue())
juliabitmap = bitmap.bitmap(0, 0, julia, height=10)

c = canvas.canvas()
c.insert(juliabitmap)
c.writeEPSfile()
c.writePDFfile()
c.writeSVGfile()



#!/usr/bin/env python3
import sys;sys.path.insert(0, "..")
from math import *
from pyx import *
# File for invalid parametrisations of Bezier curves
# Draws a sketch of the areas where invalid params are to be expected

xmin, xmax = -2, 5
ymin, ymax = -xmax, 2

g = graph.graphxy(width=10,
    x=graph.axis.lin(title=r"$\Delta x$", min=xmin, max=xmax),
    y=graph.axis.lin(title=r"$\Delta y$", min=ymin, max=ymax),
    key=graph.key.key(pos="tr"),
)
g.plot(graph.data.function("y(x)=-1", title=None), [graph.style.line([style.linestyle.dotted])])
d1 = g.plot(graph.data.function("x(y)=(-1-2*y-sqrt((1+2*y)**2+3))/3.0", title=r"cusp ($-$)"), [graph.style.line()])
d2 = g.plot(graph.data.function("x(y)=(-1-2*y+sqrt((1+2*y)**2+3))/3.0", title=r"cusp ($+$)", max=-1), [graph.style.line([color.rgb.red])])
d3 = g.plot(graph.data.function("x(y)=1.0/(3.0*(y+1))", title=r"passes through $(0,0)$", max=-1), [graph.style.line([color.rgb.green])])
d4 = g.plot(graph.data.function("x(y)=(1+y**3)/(3.0*(1+y))", title=r"passes through $(1,0)$", max=-1), [graph.style.line([color.rgb.blue])])
g.doplot()

p = d1.path << d3.path.reversed()
p.append(path.closepath())
g.layers["filldata"].fill(p, [color.gray(0.8)])

p = d2.path << d4.path.reversed()
p.append(path.closepath())
g.layers["filldata"].fill(p, [color.gray(0.8)])

g.writePDFfile()



